C **********************************************************************
C   Analyse canonique de redondance (ACR) et des correspondances (ACC) 
C   selon les modeles lineaire et polynomial, calcules pour:     
C        Matrice reponse Y (nxp)
C        Matrice explicative X (nxm)
C   Le fichier d'entree contient une en-tete (n,p,m) 
C   suivie des matrices Y (a gauche) et X (a droite), accolees.
C
C   Des test par permutation peuvent etre effectuee a la suite
C   de l'ACR ou de l'ACC. On peut tester la signification du modele
C   lineaire et du modele polynomial, ainsi que la signification 
C   de la difference de variance expliquee entre les deux modeles.
C     
C                   Vladimir Makarenkov et Pierre Legendre, April 2000
C ----------------------------------------------------------------------
C   Canonical redundancy analysis (RDA) and canonical correspondence  
C   analysis (CCA) using the linear and polynomial models, computed for:     
C        Response matrix Y (nxp)
C        Explanatory matrix X (nxm)
C   The input file contains a header (n,p,m) 
C   followed by matrices Y (left) and X (right), side by side.
C
C   Permutation tests can be done following RDA or CCA. One can test
C   the significance of the linear and polynomial models separately, 
C   and also the significance of the difference in explained variance 
C   between the two models.
C     
C                   Vladimir Makarenkov and Pierre Legendre, April 2000
C **********************************************************************
C 3456789012345678901234567890123456789012345678901234567890123456789012
 
      INCLUDE 'Rdparam.f'
C      Parameter (nmax=100)
      Integer p,nombre(nmax),Permnumber,regtype,F01,Variable01(nmax),
     +       Variable01bis(nmax)
      Real*8 Y(nmax,nmax),Qbar(nmax,nmax),Yres(nmax,nmax),
     +       Inverse(nmax,nmax),Yhat(nmax,nmax),X(nmax,nmax)
      Real*8 Dip(nmax),Dipsq(nmax),Dipmsq(nmax),Cor(nmax,nmax),
     +       Dpj(nmax),Dpjsq(nmax),Dpjmsq(nmax),Ftot,totinert
      Real*8 sx(nmax),Fmoyen(nmax), Xprime(nmax,nmax),totinertcan
      Real*8 Lambda(nmax),SingVal(nmax),U(nmax,nmax),Uhat(nmax,nmax),ak,
     +       UL(nmax,nmax),E(nmax),Centroide(nmax,nmax)
      Real*8 V(nmax,nmax),Vhat(nmax,nmax),F(nmax,nmax),CorPol(nmax,nmax)
      Real*8 Z(nmax,nmax),vect(nmax),temp,S,S1,epsilon
      Real*8 XXX(nmax,nmax),Fip(nmax)
      Equivalence (V,F), (Xprime,V), (Inverse,Vhat)
      Character tt*10, W*10 
      Character*79 nom
      epsilon=0.0000001

C Choix de la langue de dialogue / Dialogue language
C Langue du dialogue / Dialog language: 1 = francais; 2 = English
      write(*,206)
	write(*,*) 
    2 write(*,*) 'Francais: tapez (1)'
      write(*,*) 'English:  type  (2)'
      read(*,*) langue
      if((langue.lt.1).or.(langue.gt.2)) goto 2
C For Macintosh: BBEdit is the 'creator' for output files
C      call F_Creator('R*ch')
C ecrire les resultats dans un fichier? / Write results to a file?
    4 if(langue.eq.1) write(*,115) 
      if(langue.eq.2) write(*,215)
      read(*,*) ichoix4
      write(*,*)
      if((ichoix4.lt.1).or.(ichoix4.gt.2)) goto 4
      if(ichoix4.eq.1) then
         LUT=6
         else
         LUT=7
         if(langue.eq.1) then
            write(*,*) 'Nom du fichier de sortie?'
            read(*,*) nom
            open(LUT,File=nom,Status='UNKNOWN')
            else            
            write(*,*) 'Name of the output file?'
            read(*,*) nom
            open(LUT,File=nom,Status='UNKNOWN')
            endif
         if(langue.eq.1) then
            write(LUT,106)
            else
            write(LUT,206)
            endif
         inquire(unit=LUT,name=nom)
         if(langue.eq.1) write(*,*) 'Fichier de sortie: ',nom
         if(langue.eq.2) write(*,*) 'Output file: ',nom
         endif
C
C Lecture des donnees / Read data file
      call ReadData(n,p,m,nmax,Y,X,langue,LUT)
      if(LUT.eq.7) write(LUT,*)
      
C Preliminaires / Preliminaries
C Methode d'ordination / Ordination method
    6 if(langue.eq.1) write(*,116) 
      if(langue.eq.2) write(*,216)
      read(*,*) irdacca
      if((irdacca.lt.1).or.(irdacca.gt.2)) goto 6
      if(LUT.eq.6) write(LUT,*)
C     Combien de valeurs propres? / How many eigenvalues? 
      if(irdacca.eq.1) nvalc=p
      if(irdacca.eq.2) nvalc=p
      
      if(langue.eq.1) write(LUT,110) nvalc
      if(langue.eq.2) write(LUT,210) nvalc

      if(irdacca.eq.1) nvalpr=min(p,(n-1))
      if(irdacca.eq.2) nvalpr=min((p-1),(n-1))
      nvalpr=p
     
      if(langue.eq.1) write(LUT,111) nvalpr
      if(langue.eq.2) write(LUT,211) nvalpr
      if(LUT.eq.7) write(LUT,*)

C     Analyse canonique et/ou analyse des residus ?
C     Canonical analysis and/or analysis of residuals ?
      if(m.gt.0) then
   10    if(langue.eq.1) write(*,113) 
         if(langue.eq.2) write(*,213)
         read(*,*) ichoix2
         if((ichoix2.lt.1).or.(ichoix2.gt.2)) goto 10
         endif
C     Impression de resultats intermediaires ?
C     Print intermediate result tables ?
C   12 if(langue.eq.1) write(*,109) 
C      if(langue.eq.2) write(*,209)
C      read(*,*) iprint
C      if((iprint.lt.0).or.(iprint.gt.4)) goto 12
       iprint=0
C     Cadrage des lignes et des colonnes
C     Scaling of rows and columns
   14 if(irdacca.eq.1) then
         if(langue.eq.1) write(*,117) 
         if(langue.eq.2) write(*,217)
         else
         if(langue.eq.1) write(*,114) 
         if(langue.eq.2) write(*,214)
         endif
      read(*,*) ichoix3
      if((ichoix3.lt.1).or.(ichoix3.gt.2)) goto 14
      if(LUT.eq.6) write(LUT,*)
C     Demarrage de l'horloge / Start the clock
C      call time(tt)
      debut=secnds(0.0)
      
C Detection des variables binaires dans X 
C Detection of binary variables in X
      F01=m-1
      do 16 j=1,m-1
      Variable01(j)=1
      do 15 i=1,n
C      if (.not.(X(i,j).eq.1.0).or.(X(i,j).eq.0.0)) then
C To detect binary variables: if y = 0 or 1, then y*y - y = 0
      if (dabs(X(i,j)*X(i,j)-X(i,j)).gt.epsilon) then
         Variable01(j)=0
         F01=F01-1
         go to 16
      end if
   15 continue
      write(*,*) 'Variable ',j,' of matrix X is binary'
   16 continue
C      write(*,*) 'Vector Variable01:'
C      write(*,*) (Variable01(j), j=1,m-1)
      
      ichoix6=0
      if (F01.gt.0) then
   17      if(langue.eq.1) write(*,120) 
           if(langue.eq.2) write(*,220)      
           read(*,*) ichoix6
           if((ichoix6.lt.1).or.(ichoix6.gt.2)) goto 17
           write(*,*) 
      endif
      
      do 422 j=1,m-1
  422 Variable01bis(j)=Variable01(j)
      if (ichoix6.eq.1) then
         do 424 j=1,m-1
  424    Variable01(j)=0
      endif
   
C En ACR: transformation des variables Y
C In RDA: transformation of the Y variables      
           
      if (irdacca.eq.1) then
   18 write(*,*)
      if(langue.eq.1) then 
        write(*,*) 'Transformation des variables Y avant l''analyse ?'
        write(*,*) '  (0)  Aucune transformation'
        write(*,*) '  (1)  Transformation Y = Log(Y + 1), log naturel'
        write(*,*) '  (2)  Transformation Y = exp(Y)-1'
      else 
        write(*,*) 'Transform the Y variables before analysis ?'
        write(*,*) '  (0)  No transformation'
        write(*,*) '  (1)  Transformation Y = Log(Y + 1), natural log'
        write(*,*) '  (2)  Transformation Y = exp(Y)-1'
      end if
         read(*,*) Transformation
         if ((Transformation.lt.0).or.(Transformation.gt.2)) go to 18  
      
         if (Transformation.eq.1) then
            do 19 i=1,n         
            do 19 j=1,p
            Y(i,j)=dlog(Y(i,j)+1.0)
   19       continue  
         endif              

         if (Transformation.eq.2) then
            do 20 i=1,n         
            do 20 j=1,p
            Y(i,j)=dexp(Y(i,j))-1
   20       continue  
         endif
      endif
      
C Transformer les matrices Y et X avant l'ACR ou l'ACC 
C Transform matrices Y and X prior to RDA or CCA
              
      if(irdacca.eq.1) then
                  
         do 22 i=1,n
         Dip(i)=1.0
   22    Dipsq(i)=1.0
         
         call TransRDA(n,p,m,nmax,Y,X,Variable01,temp,
     +                 sx,iprint,langue,LUT,1)
C        Y est maintenant centre / Y is now centred
         totinert=temp/dfloat(n-1)
         if(langue.eq.1) write(LUT,102) temp,totinert
         if(langue.eq.2) write(LUT,202) temp,totinert
         
      else
         
         call TransCCA(n,p,m,nmax,Y,Qbar,X,Variable01,Fip,Dip,Dpj,Ftot,
     +                 totinert,sx,iprint,langue,LUT)

C        ACC: Y contient maintenant les frequences relatives
C        CCA: Y now contains relative frequencies

         do 24 i=1,n
         Dipsq(i)=dsqrt(Dip(i))
   24    Dipmsq(i)=1.0/Dipsq(i)
         do 26 j=1,p
         Dpjsq(j)=dsqrt(Dpj(j))
   26    Dpjmsq(j)=1.0/Dpjsq(j)
         if(langue.eq.1) write(LUT,101) totinert
         if(langue.eq.2) write(LUT,201) totinert
      endif
          
C Regression lineaire ou polynomiale/ Linear or polynomial regression
C TransRDA is called here only to centre Yhat and compute inertia

      if(irdacca.eq.1) then

         call Regression(X,Y,Yhat,n,p,m,regtype,Variable01,irdacca,
     +                   Dip,LUT,langue) 
         call TransRDA(n,p,m,nmax,Yhat,X,Variable01,temp,
     +                 sx,iprint,langue,LUT,2)
         totinertcan=temp/dfloat(n-1)

         if(langue.eq.1) write(LUT,*) 'Analyse canonique: '
         if(langue.eq.2) write(LUT,*) 'Canonical analysis:'
         
         if(langue.eq.1) write(LUT,102) temp,totinertcan
         if(langue.eq.2) write(LUT,202) temp,totinertcan
         
         do 30 i=1,n
         do 30 j=1,p        
   30    Yres(i,j)=Y(i,j)-Yhat(i,j)
   
      else
                 
   	  call Regression(X,Qbar,Yhat,n,p,m,regtype,Variable01,irdacca,
     +                   Dip,LUT,langue)       
        
         do 32 i=1,n
         do 32 j=1,p                
   32    Yres(i,j)=Qbar(i,j)-Yhat(i,j)   	
      endif	  
      
      if((iprint.eq.2).or.(iprint.eq.4)) then
         if(langue.eq.1) write(LUT,*) 
     +         'Matrice des valeurs prevues par les regressions)'
         if(langue.eq.2) write(LUT,*) 'Matrix of fitted values)'
         write(LUT,*)
         do 34 i=1,n
   34    write(LUT,100) (Yhat(i,j), j=1,p)
         write(LUT,*)
         if(langue.eq.1) write(LUT,*) 'Matrice des residus'
         if(langue.eq.2) write(LUT,*) 'Matrix of residuals'
         write(LUT,*)
         do 36 i=1,n
   36    write(LUT,100) (Yres(i,j), j=1,p)
         write(LUT,*)
      endif
C Analyse canonique / Canonical analysis
C      if((ichoix2.ne.1).and.(ichoix2.ne.2)) goto 40
      if(irdacca.eq.1) then
         if(langue.eq.1) then
            write(LUT,*) '*** Analyse canonique de redondance ***'
            else
            write(LUT,*) '*** Canonical redundancy analysis ***'
            endif
         else
         if(langue.eq.1) then
            write(LUT,*) '*** Analyse canonique des correspondances ***'
            else
            write(LUT,*) '*** Canonical correspondence analysis ***'
            endif
         endif
      ican=1
      call Eigen(n,p,nmax,langue,irdacca,ican,nombre,nvalc,
     +           Yhat,totinert,Lambda,SingVal,U,UL,E,Z,vect,LUT,temp)
      if(temp.eq.0.0) goto 40
      if(irdacca.eq.1) then
      call ScaleRDA(n,p,nmax,langue,ichoix3,ican,nvalc,Y,Yhat,
     +           Lambda,SingVal,U,F,Z,vect,LUT,regtype)
      else
      call ScaleCCA(n,p,nmax,langue,ichoix3,ican,nvalc,Qbar,Yhat,
     + Dipmsq,Dpjmsq,Lambda,SingVal,U,Uhat,UL,V,Vhat,Z,iprint,LUT,
     + regtype)
      endif

      if (regtype.eq.2) then
      if(langue.eq.1) write(LUT,*) 'On peut employer l''un ou ',
     +  'l''autre des ensembles de vecteurs ci-dessous ',
     +  'pour representer les variables explicatives X dans le ',
     +  'diagramme de double projection'
      if(langue.eq.2) write(LUT,*) 'Either of the two following sets ',
     +  'can be used to represent the explanatory ',
     +  'variables X in the biplot'
      end if
       
C Position des variables X dans le diagramme de double projection
C Biplot scores of the X variables
       if (regtype.eq.1) then       
       
       if (irdacca.eq.1) then
           call Correlation(Z,X,Cor,n,nvalc,m-1)
       else
           call CorrelationPonderee(Z,X,Dip,Cor,n,nvalc,m-1) 
       endif
        
       if(langue.eq.1) then 
          write(LUT,122) 
       else       
          write(LUT,222)           
       end if
       
       do 52 i=1,m-1
        if (ichoix4.eq.1) then  
          if (i.lt.10) then
              write(LUT, '(A2,I1,A1,$)') 'X',i,' '
          else if (i.lt.100) then 
              write(LUT, '(A2,I2,$)') 'X',i
          else 
              write(LUT, '(A2,I4,$)') 'X',i            
          end if
        else 
          if (i.lt.10) then
              write(LUT, '(A1,I1,A1,$)') 'X',i,' '
          else if (i.lt.100) then 
              write(LUT, '(A1,I2,$)') 'X',i
          else 
              write(LUT, '(A1,I4,$)') 'X',i            
          end if        
        end if    
          
       if (ichoix3.eq.1) then              
        if (nvalc.le.7) then
         write(LUT,100) 
     +     (Cor(i,j)*sqrt(Lambda(j)/totinert), j=1,nvalc)
        else 
         write(LUT,100) 
     +     (Cor(i,j)*sqrt(Lambda(j)/totinert), j=1,7)
         write(LUT,400) 
     +     (Cor(i,j)*sqrt(Lambda(j)/totinert), j=8,nvalc)    
         end if                           
       else      
        if (nvalc.le.7) then
         write(LUT,100) (Cor(i,j), j=1,nvalc)     
        else 
         write(LUT,100) (Cor(i,j), j=1,7)
         write(LUT,400) (Cor(i,j), j=8,nvalc)    
        end if                    
       endif
     
       
   52  continue      
       write(LUT,*)
             
       else 
              
       call Correlation_Polyn_Pond(Z,X,Dip,CorPol,n,nvalc,m-1,
     + Variable01)
       if(langue.eq.1) then 
          write(LUT,121) 
       else       
          write(LUT,221)           
       end if
       
       do 155 i=1,m-1
          if (ichoix4.eq.1) then  
           if (i.lt.10) then
              write(LUT, '(A2,I1,A1,$)') 'X',i,' '
           else if (i.lt.100) then 
              write(LUT, '(A2,I2,$)') 'X',i
           else 
              write(LUT, '(A2,I4,$)') 'X',i   
           end if 
          else
           if (i.lt.10) then
              write(LUT, '(A1,I1,A1,$)') 'X',i,' '
           else if (i.lt.100) then 
              write(LUT, '(A1,I2,$)') 'X',i
           else 
              write(LUT, '(A1,I4,$)') 'X',i   
           end if 
          end if
               
       if (ichoix3.eq.1) then              
        if (nvalc.le.7) then
         write(LUT,100) 
     +     (CorPol(i,j)*sqrt(Lambda(j)/totinert), j=1,nvalc)
        else 
         write(LUT,100) 
     +     (CorPol(i,j)*sqrt(Lambda(j)/totinert), j=1,7)
         write(LUT,500) 
     +     (CorPol(i,j)*sqrt(Lambda(j)/totinert), j=8,nvalc)    
         end if                           
       else      
        if (nvalc.le.7) then
         write(LUT,100) (CorPol(i,j), j=1,nvalc)     
        else 
         write(LUT,100) (CorPol(i,j), j=1,7)
         write(LUT,500) (CorPol(i,j), j=8,nvalc)    
        end if                    
       endif
       
  155  continue
       write(*,*)
             
       end if
       
C Autre facon de calculer la position des variables X dans
C le diagramme de double projection: correlations avec les variables
C simples, au carre et produits.
C Alternative procedure: compute biplot scores for the simple variables,
C the squared variables and the product variables 

      if (regtype.eq.2) then
          mm=m-1+m-1+(m-1)*(m-2)/2
          if (mm.gt.nmax) then
             mm=nmax
             if(langue.eq.1) then
                write(LUT,*) 'Seulement', nmax,' coordonnees',
     +          ' des variables explicatives ont ete calculees.'
                write(LUT,*) 'Pour calculer plus de coordonnees,',
     +          ' augmenter le parametre nmax dans'
                write(LUT,*) ' le fichier RDA_CCA-parameters.f',
     +          ' fourni avec le programme et recompilez-le.' 
             else 
               write(LUT,*) 'Only', nmax,' biplot scores have been',
     +         ' computed.'
               write(LUT,*) 'To compute more, increase the value of',
     +         ' parameter nmax in'
               write(LUT,*) 'the file RDA_CCA-parameters.f',
     +         ' provided with the programm and recompile it.'               
             end if
             write(LUT,*)
          end if  
       
          if (mm.gt.(2*m-2)) mmm=2*m-2
          if (mm.LE.(2*m-2)) mmm=mm
       
          do 453 j=1,mmm
          do 453 i=1,n
          if (j.LE.m-1) then
             XXX(i,j)=X(i,j)
          else
             kk=j-(m-1)
             XXX(i,j)=X(i,kk)*X(i,kk)
          end if              
  453     continue
     
          do 454 j1=1,m-1
          do 454 j2=j1+1,m-1 
          do 454 i=1,n
          jj=2*m-2+(m-1)*(j1-1)-j1*(j1-1)/2+j2-j1
            if (jj.LE.mm) then
              XXX(i,jj)=X(i,j1)*X(i,j2)
            else 
              go to 459
            end if
  454     continue 
  459     continue
         
          call CorrelationPonderee(Z,XXX,Dip,Cor,n,nvalc,mm)
        
          if(langue.eq.1) then 
                write(LUT,123) 
          else       
                write(LUT,223)           
          end if
          
              
           do 465 i=1,2*m-2                
           if (i.gt.mm) go to 460
         
          if (ichoix4.eq.1) then  
           if (i.LE.m-1) then      
             if (i.lt.10) then
                write(LUT, '(A2,I1,A5,$)') 'X',i,'     '
             else if (i.lt.100) then 
                write(LUT, '(A2,I2,A4,$)') 'X',i,'    '
             else if (i.lt.1000) then 
                write(LUT, '(A2,I3,A3,$)') 'X',i,'   '
             else 
                write(LUT, '(A2,I5,A2,$)') 'X',i,'  '   
             end if
           else if (i.LE.2*m-2) then
             if ((i-(m-1)).lt.10) then
                write(LUT, '(A2,I1,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I1,A2,$)') 'X',i-(m-1),'  '                
             else if (i-(m-1).lt.100) then 
                write(LUT, '(A2,I2,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I2,$)') 'X',i-(m-1)
             else if (i-(m-1).lt.1000) then 
                write(LUT, '(A2,I3,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I3,$)') 'X',i-(m-1)  
             else 
                write(LUT, '(A2,I5,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I5,$)') 'X',i-(m-1) 
             end if    
           end if
          else
             if (i.LE.m-1) then      
             if (i.lt.10) then
                write(LUT, '(A1,I1,A5,$)') 'X',i,'     '
             else if (i.lt.100) then 
                write(LUT, '(A1,I2,A4,$)') 'X',i,'    '
             else if (i.lt.1000) then 
                write(LUT, '(A1,I3,A3,$)') 'X',i,'   '
             else 
                write(LUT, '(A1,I5,A2,$)') 'X',i,'  '   
             end if
           else if (i.LE.2*m-2) then
             if ((i-(m-1)).lt.10) then
                write(LUT, '(A1,I1,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I1,A2,$)') 'X',i-(m-1),'  '                
             else if (i-(m-1).lt.100) then 
                write(LUT, '(A1,I2,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I2,$)') 'X',i-(m-1)
             else if (i-(m-1).lt.1000) then 
                write(LUT, '(A1,I3,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I3,$)') 'X',i-(m-1)  
             else 
                write(LUT, '(A1,I5,A1,$)') 'X',i-(m-1),'*'
                write(LUT, '(A1,I5,$)') 'X',i-(m-1) 
             end if    
           end if
          end if
                    
           if (ichoix3.eq.1) then              
           if (nvalc.le.7) then
             write(LUT,100) 
     +       (Cor(i,j)*sqrt(Lambda(j)/totinert), j=1,nvalc)
           else 
             write(LUT,100) 
     +       (Cor(i,j)*sqrt(Lambda(j)/totinert), j=1,7)
           write(LUT,400) 
     +       (Cor(i,j)*sqrt(Lambda(j)/totinert), j=8,nvalc)    
           end if                           
           else      
           if (nvalc.le.7) then
             write(LUT,100) (Cor(i,j), j=1,nvalc)     
           else 
             write(LUT,100) (Cor(i,j), j=1,7)
             write(LUT,400) (Cor(i,j), j=8,nvalc)    
           end if                    
          endif                      
         
  465      continue        
           
           do 452 j1=1,m-1
           do 452 j2=j1+1,m-1
                      
             i=2*m-2+(m-1)*(j1-1)-j1*(j1-1)/2+j2-j1         
             if (i.gt.mm) go to 460 
             
            if (ichoix4.eq.1) then             
             if (j1.lt.10) then
                write(LUT, '(A2,I1,A1,$)') 'X',j1,'*'
             else if (j1.lt.100) then 
                write(LUT, '(A2,I2,A1,$)') 'X',j1,'*'
             else if (j1.lt.1000) then 
                write(LUT, '(A2,I3,A1,$)') 'X',j1,'*'  
             else 
                write(LUT, '(A2,I5,A1,$)') 'X',j1,'*'
             end if 
                          
             if (j2.lt.10) then
                if (j1.lt.10)   write(LUT, '(A1,I1,A2,$)') 'X',j2,'  '
                if ((j1.gt.9).and.(j1.lt.100)) 
     +              write(LUT, '(A1,I1,A1,$)') 'X',j2,' '
                if (j1.gt.99)   write(LUT, '(A1,I1,$)') 'X',j2
             else if (j2.lt.100) then 
                if (j1.lt.10)   write(LUT, '(A1,I2,A1,$)') 'X',j2,' '
                if (j1.gt.9)    write(LUT, '(A1,I2,$)') 'X',j2
             else if (j2.lt.1000) then 
                write(LUT, '(A1,I3,$)') 'X',j2  
             else 
                write(LUT, '(A1,I5,$)') 'X',j2                   
             end if 
            else 
             if (j1.lt.10) then
                write(LUT, '(A1,I1,A1,$)') 'X',j1,'*'
             else if (j1.lt.100) then 
                write(LUT, '(A1,I2,A1,$)') 'X',j1,'*'
             else if (j1.lt.1000) then 
                write(LUT, '(A1,I3,A1,$)') 'X',j1,'*'  
             else 
                write(LUT, '(A1,I5,A1,$)') 'X',j1,'*'
             end if 
                          
             if (j2.lt.10) then
                if (j1.lt.10)   write(LUT, '(A1,I1,A2,$)') 'X',j2,'  '
                if ((j1.gt.9).and.(j1.gt.100)) 
     +              write(LUT, '(A1,I1,A1,$)') 'X',j2,' '
                if (j1.gt.99)   write(LUT, '(A1,I1,$)') 'X',j2
             else if (j2.lt.100) then 
                if (j1.lt.10)   write(LUT, '(A1,I2,A1,$)') 'X',j2,' '
                if (j1.gt.9)    write(LUT, '(A1,I2,$)') 'X',j2
             else if (j2.lt.1000) then 
                write(LUT, '(A1,I3,$)') 'X',j2  
             else 
                write(LUT, '(A1,I5,$)') 'X',j2                   
             end if 
            end if              
                 
            if (ichoix3.eq.1) then              
             if (nvalc.le.7) then
              write(LUT,100) 
     +        (Cor(i,j)*sqrt(Lambda(j)/totinert), j=1,nvalc)
             else 
               write(LUT,100) 
     +         (Cor(i,j)*sqrt(Lambda(j)/totinert), j=1,7)
               write(LUT,400) 
     +         (Cor(i,j)*sqrt(Lambda(j)/totinert), j=8,nvalc)    
             end if                           
            else      
             if (nvalc.le.7) then
               write(LUT,100) (Cor(i,j), j=1,nvalc)     
             else 
               write(LUT,100) (Cor(i,j), j=1,7)
               write(LUT,400) (Cor(i,j), j=8,nvalc)    
             end if                    
            endif              
             
  452      continue      
  460      write(LUT,*)       
       end if
      
       
C Calculer les coordonnes des centroides       
       if (F01.gt.0) then
       if(langue.eq.1) then
        write(LUT,*) 
     +  'Coordonnees des centroides des variables explicatives binaires'
       else 
        write(LUT,*)
     +  'Biplot scores of centroids of binary explanatory variables'  
       end if
       
       write(LUT,*)
       do 150 i=1,m-1
       if (Variable01bis(i).eq.1) then
           do 151 j=1,nvalc
           Centroide(i,j)=0.0
           ak=0.0
           do 152 i1=1,n
C          if (X(i1,i).eq.1.0) then
           if (X(i1,i).GE.0.0) then                                          
              ak=ak+Dip(i1)
              Centroide(i,j)=Centroide(i,j)+Z(i1,j)*Dip(i1)
           end if   
  152      continue
           Centroide(i,j)=Centroide(i,j)/ak
          
  151      continue
                     
         if (ichoix4.eq.1) then
          if (i.lt.10) then
              write(LUT, '(A2,I1,A1,$)') 'X',i,' '
          else if (i.lt.100) then 
              write(LUT, '(A2,I2,$)') 'X',i
          else 
              write(LUT, '(A2,I4,$)') 'X',i   
          end if 
         else 
          if (i.lt.10) then
              write(LUT, '(A1,I1,A1,$)') 'X',i,' '
          else if (i.lt.100) then 
              write(LUT, '(A1,I2,$)') 'X',i
          else 
              write(LUT, '(A1,I4,$)') 'X',i   
          end if      
         end if 
           
          if (nvalc.le.7) then
             write(LUT,100) (Centroide(i,j), j=1,nvalc)     
          else 
             write(LUT,100) (Centroide(i,j), j=1,7)
             write(LUT,500) (Centroide(i,j), j=8,nvalc)    
          end if                       
         
       end if 
  150  continue
       end if 
       
       write(*,*) 
  149  if(langue.eq.1) then
          write(*,*) 'Tests de signification:'
          write(*,*) '  (0) Pas de test par permutation'
          write(*,*) '  (1) Test par permutation pour la signification',
     +                      ' des analyses canoniques'
       else 
          write(*,*) 'Tests of significance:'
          write(*,*) '  (0) No permutation test'               
          write(*,*) '  (1) Permutation tests for the significance',
     +                      ' of the canonical analyses'
       end if

       read(*,*)    itest
       if((itest.lt.0).or.(itest.gt.1)) goto 149        
       
       if (itest.eq.1) then       
       
  129  write(*,*)
       if(langue.eq.1) then       
          write(*,*) 'Combien de permutations? E.g. 499, 999, ...',
     +               ' (maximum 9999)'
       else
          write(*,*) 'How many permutations? E.g. 499, 999, ...',
     +               ' (maximum 9999)'
       end if
              
       read(*,*)    Permnumber
       if((Permnumber.lt.1).or.(Permnumber.gt.9999)) goto 129
       Permnumber=Permnumber+1 
       write(LUT,*) 
          
                     
       if (irdacca.eq.1) then
          call PermutationTest(Y,X,Fmoyen,n,p,m,Dip,irdacca,Permnumber,
     +                         Variable01,langue,LUT)
       else
          call PermutationTest(Qbar,X,Fmoyen,n,p,m,Dip,irdacca,
     +                         Permnumber,Variable01,langue,LUT)
       endif
                     
      end if
      
C Analyse de residus (Yres); ACP ou AFC
C Analysis of residuals (Yres); PCA or CA
   40 if((ichoix2.ne.2)) goto 99
      write(LUT,*)
   42 if(m.eq.0) then
        if(irdacca.eq.1) then
         if(langue.eq.1) then
            write(LUT,*) '*** Analyse en composantes principales ***'
            else
            write(LUT,*) '*** Principal component analysis ***'
            endif
        else
         if(langue.eq.1) then
            write(LUT,*) '*** Analyse des correspondances ***'
            else
            write(LUT,*) '*** Correspondence analysis ***'
            endif
        endif
      else
      if(langue.eq.1) write(LUT,*) '*** Analyse des residus ***'
      if(langue.eq.2) write(LUT,*) '*** Analysis of residuals ***'
      endif
      ican=2
      call Eigen(n,p,nmax,langue,irdacca,ican,nombre,nvalpr,
     +           Yres,totinert,Lambda,SingVal,U,UL,E,Z,vect,LUT,temp)
      if(temp.eq.0.0) goto 99
      if(irdacca.eq.1) then
      call ScaleRDA(n,p,nmax,langue,ichoix3,ican,nvalpr,Y,Yres,
     +           Lambda,SingVal,U,F,Z,vect,LUT,regtype)
      else
      call ScaleCCA(n,p,nmax,langue,ichoix3,ican,nvalpr,Qbar,Yres,
     +     Dipmsq,Dpjmsq,Lambda,SingVal,U,Uhat,UL,V,Vhat,Z,iprint,
     +     LUT,regtype)
      endif
C ----------------------------------------------------------------------
C   99 call time(tt)
C      fin=secnds(debut)
C      if(langue.eq.1) write(LUT,108) fin
C      if(langue.eq.2) write(LUT,208) fin
   99  write(*,*)
       if(langue.eq.1) write(*,*) 'Fin du programme. '
	 if(langue.eq.2) write(*,*) 'End of the program. '
      write(*,*) 
      if(langue.eq.1) then
            write(*,*) 'Voulez-vous faire une autre analyse? (O/N)'
            read(*,*) W
      else            
            write(*,*) 'Another analysis? (Y/N)'
            read(*,*) W
      endif
      if ((W.eq.'O').or.(W.eq.'o').or.(W.eq.'Y').or.(W.eq.'y')) then 
         goto 4
      else
         stop
      endif
C **********************************************************************
  100 format(8F10.5)
  400 format(T8,7F10.5)
  500 format(T4,7F10.5)
  101 format(' Inertie totale (variance totale de l''AC) =',F10.5/)
  201 format(' Total inertia (total CA variance) =',F10.5/)
  102 format(' Inertie totale = ',f15.5,'   Variance totale = ',f15.5/)
  202 format(' Total inertia = ',f15.5,'   Total variance = ',f15.5/)
  106 format(/' Analyse canonique de redondance (ACR)'/
     + ' et analyse canonique des correspondances (ACC)'/
     + ' selon les modeles lineaire et polynomial,'/
     + ' avec tests par permutation.'//
     + ' Vladimir Makarenkov et Pierre Legendre'/
     + ' Departement de sciences biologiques'/
     + ' Universite de Montreal.'//
     + '            (c) V. Makarenkov & P. Legendre, 2000'//
     + 1x,46('-'))
  206 format(/' Canonical redundancy analysis (RDA)'/
     + ' and canonical correspondence analysis (CCA)'/
     + ' using the linear and polynomial models,'/
     + ' with permutation tests.'//
     + ' Vladimir Makarenkov and Pierre Legendre'/
     + ' Departement de sciences biologiques'/
     + ' Universite de Montreal.'//
     + '         (c) V. Makarenkov & P. Legendre, 2000'//
     + 1x,43('-'))
  108 format(/' Temps de calcul:',F10.2,' secondes')
  208 format(/' Time spent:',F10.2,' seconds')
  109 format(/' Impression des resultats intermediaires?'/
     + '   (0) Aucun resultat intermediaire'/ 
     + '   (1) Matrices Y centree (ou Q-bar, en ACC) et X centree'/
     + '   (2) Matrices des valeurs prevues par les regressions',
     + ' et des residus'/
     + '   (3) Matrices U et U^ (en AFC-ACC)'/
     + '   (4) Toutes ces matrices')
  209 format(/' Print intermediate calculation results?'/
     + '   (0) None'/ 
     + '   (1) Matrices Y centred (or Q-bar, in CCA) and X centred,'/
     + '   (2) Matrices of values fitted to regressions and residuals'/
     + '   (3) Matrices U and U^ (in CA-CCA)'/
     + '   (4) All of these matrices')
  110 format(' Nombre max. de valeurs propres canoniques =',i3)
  210 format(' Maximum number of canonical eigenvalues =',i3)
  111 format(' Nombre max. de valeurs propres non canoniques =',i3)
  211 format(' Maximum number of non-canonical eigenvalues =',i3)
  113 format(/' Calculs a realiser: '/
     + '   (1) Analyse canonique'/
     + '   (2) Analyse canonique + analyse de la matrice des residus')
C     + '   (3) Les deux')
  213 format(/' Compute the following: '/
     + '   (1) Canonical analysis'/
     + '   (2) Canonical analysis + analysis of the residual matrix')
C     + '   (3) Both')
  114 format(/' Cadrage des lignes et des colonnes:'/
     + '   (1) Lignes-sites (mat. F)',
     +       ' au centroide des colonnes-especes (mat. V)'/
     + '   (2) Colonnes-especes (mat. F-hat)',
     +       ' au centroide des lignes-sites (mat. V-hat)')
  214 format(/' Scaling of rows and columns:'/
     + '   (1) Row-sites (mat. F) at centroids',
     +       ' of column-species (mat. V)'/
     + '   (2) Column-species (mat. F-hat) at centroids',
     +       ' of row-sites (mat. V-hat)')
  115 format(/' Ecrire les resultats'/
     + '   (1) A l''ecran'/
     + '   (2) Dans un fichier')
  215 format(/' Write results'/
     + '   (1) To the screen'/
     + '   (2) To an output file')
  116 format(/' Methode d''ordination:'/
     + '   (1) ACR (analyse canonique de redondance)'/
     + '   (2) ACC (analyse canonique des correspondances)')
  216 format(/' Ordination method:'/
     + '   (1) RDA (redundancy analysis)'/
     + '   (2) CCA (canonical correspondence analysis)')
  117 format(/' Cadrage des lignes et des colonnes:'/
     + '   (1) Distances entre sites preservees'/
     + '   (2) Correlations entre variables preservees')
  217 format(/' Scaling of rows and columns:'/
     + '   (1) Distance biplot: distances among sites',
     +       ' approximate Euclidean distances'/
     + '   (2) Correlation biplot: angles reflect the',
     +       ' correlations among variables')
  
  120 format(/' Transformation des variables binaires:'
     +       /'   (1) Centrer les variables binaires qui forment X'
     +       /'   (2) Ne pas centrer les variables binaires de X')
     
  220 format(/' Transformation of the binary variables:'
     +       /'   (1) Center the binary variables found in X'
     +       /'   (2) Do not centre the binary variables in X')

  121 format(/' PREMIERE FAcON DE REPRESENTER LES VARIABLES',
     +        ' EXPLICATIVES:'
     +       /' Positions des variables explicatives dans le diagramme',
     +        ' de double projection' 
     +       /' (''biplot diagram''); correlations lineaires',
     +        ' multiples entre les variables'
     +       /' explicatives X et les coordonnees des sites Z'/) 
  
  221 format(/' FIRST WAY OF REPRESENTING EXPLANATORY VARIABLES',
     +        ' IN BIPLOT:'
     +       /' Biplot scores of explanatory variables; multiple',
     +        ' linear correlations'
     +       /' between the explanatory variables X and the site',
     +        ' scores Z'/)
  
  122 format(/' Positions des variables explicatives dans le diagramme',
     +        ' de double projection'
     +       /' (''biplot diagram''); correlations lineaires entre',
     +        ' les variables'
     +       /' explicatives X et les coordonnees des sites Z'/)  
                      
  222 format(/' Biplot scores of explanatory variables; linear',
     +        ' correlations between'
     +       /' the explanatory variables X and the site scores Z'/) 
  
  123 format(/' DEUXIEME FAcON DE REPRESENTER LES VARIABLES',
     +        ' EXPLICATIVES:'
     +       /' Positions des variables explicatives X, des carres de',
     +        ' ces variables X^2 et'
     +       /' des produits combines Xi*Xj (i,j=1,...,m) dans le',
     +        ' diagramme de double projection'
     +       /' (''biplot diagram''), calculees a l''aide des',
     +        ' correlations lineaires'
     +       /'  entre X, X^2, Xi*Xj et les coordonnees des sites Z'/) 
                      
  223 format(/' SECOND WAY OF REPRESENTING EXPLANATORY VARIABLES',
     +        ' IN BIPLOT:'
     +       /' Biplot scores of explanatory variables X,their',
     +        ' squares X^2 and '
     +       /' products Xi*Xj (i,j=1,...,m); linear correlations',
     +        ' between the explanatory'
     +       /' variables X, X^2, and Xi*Xj and the site scores Z'/) 
                    
      end
      

      Subroutine Correlation(Z,X,Cor,n,nvalc,m)
C Calcul de la matrice des coefficients de correlation Cor(m,nvalc) 
C entre les matrices Z(nxnvalc) et X(nxm). 
C       
C Compute the matrix of correlation coefficients Cor(m,nvalc) 
C between matrices Z(nxnvalc) and X(nxm).        
      INCLUDE 'Rdparam.f'
      Parameter (nmax1=nmax)
      Integer n,nvalc,m,i,j,k
      Real*8 Z(nmax,nmax),X(nmax,nmax),Cor(nmax,nmax),
     +       MoyZ(nmax1),MoyX(nmax1)
     
      do 5 i=1,nvalc
      MoyZ(i)=0.0
      do 5 k=1,n
   5  MoyZ(i)=MoyZ(i)+Z(k,i)/n
      do 6 i=1,m
      MoyX(i)=0.0
      do 6 k=1,n
   6  MoyX(i)=MoyX(i)+X(k,i)/n
      
      do 10 i=1,nvalc
      do 10 j=1,m
      S=0.0
      S1=0.0
      S2=0.0
      do 20 k=1,n
      S=S+(Z(k,i)-MoyZ(i))*(X(k,j)-MoyX(j))
      S1=S1+(Z(k,i)-MoyZ(i))**2
  20  S2=S2+(X(k,j)-MoyX(j))**2
      Cor(j,i)=S/(sqrt(S1*S2))
  10  continue
      return
      end


      Subroutine Correlation_Polynomiale(Y,X,CorPol,n,p,m)
C Calcul la matrice des coefficients de correlation multiple CorPol(m,p) 
C entre les matrices Y(nxp) et X(nxm). Pour chaque variable y de Y et
C chaque variable x de X, on calcule la correlation lineaire multiple 
C R(y,x) entre les vecteurs et (x et x^2). Le signe de la correlation
C lineaire simple entre y et x est attribue a R(y,x) pour obtenir 
C la valeur finale CorPol(y,x).
C La matrice des correlations lineaires multiples CorPol est employee 
C pour representer les variables explicatives de X dans les diagrammes
C de double projection en ACR polynomiale. 
C
C Calculation of the matrix of multiple correlation coefficients 
C Cor(m,p) between matrices Y(nxp) and X(nxm). For each variable y of Y
C and each variable x of X, the multiple linear correlation R(y,x) 
C is computed between vectors y and (x and x^2). The sign of the simple
C linear correlation between y and c is assigned to R(y,x) to obtain the
C final value CorPol(y,x). The matrix of the multiple linear
C correlations CorPol is used in polynomial RDA to represent
C the explanatory variables in X in the biplot diagram.                
C
      INCLUDE 'Rdparam.f'
      Parameter (nmax1=nmax)
      Integer n,p,m,i,j,j1,m1
      Real*8 Y(nmax,nmax),X(nmax,nmax),CorPol(nmax,nmax),S,S1,S2,
     +       Xtr(nmax1,nmax1),Yhat(nmax1,nmax1),CorLin(nmax1,nmax1),
     +       Ymoy(nmax1),B(nmax1,nmax1)
     
      call Correlation(Y,X,CorLin,n,p,m)            
      do 10 j=1,m
      m1=3
      S1=0.0
      do 15 i=1,n
      S1=S1+dabs(X(i,j)-X(i,j)**2)
   15 continue  
      if (S1.LE.0.01) m1=2
      do 20 i=1,n
      Xtr(i,1)=X(i,j)
      if (m1.eq.3) then
          Xtr(i,2)=X(i,j)**2
          Xtr(i,3)=1.0
      else
          Xtr(i,2)=1.0
      end if
   20 continue      
      call Linreg(Xtr,Y,Yhat,B,n,p,m1)
      
      do 30 j1=1,p
      S=0.0
      do 40 i=1,n
      S=S+Y(i,j1)
   40 continue
      Ymoy(j1)=S/n
   30 continue
      do 50 j1=1,p
      S=0.0
      S2=0.0
      do 60 i=1,n
      S=S+(Y(i,j1)-Ymoy(j1))**2
      S2=S2+(Yhat(i,j1)-Ymoy(j1))**2
   60 continue 
      CorPol(j,j1)=dsqrt(S2/S)
      CorPol(j,j1)=dsign(CorPol(j,j1),CorLin(j,j1))           
   50 continue               
   10 continue
      return
      end

      
      Subroutine Correlation_Polyn_Pond(Z,X,Dip,CorPol,n,p,m,Variable01)
C Calcul de la matrice des coefficients de correlation multiple ponderes 
C CorPol(m,p) entre les matrices Z(nxp) et X(nxm), tenant compte du  
C vecteur des poids Dip(n). Pour chaque variable y de Y et chaque 
C variable x de X, on calcule la correlation lineaire multiple ponderee
C R(y,x) entre les vecteurs et (x et x^2). Le signe de la correlation 
C lineaire simple ponderee entre y et x est attribue a R(y,x) pour
C obtenir la valeur finale CorPol(y,x). La matrice des correlations  
C lineaires multiples CorPol est employee pour representer les variables
C explicatives X dans le diagramme de double projection en ACC 
C polynomiale.
C
C Calculation of the matrix of the multiple correlation coefficients 
C Cor(m,p) between matrices Y(nxp) and X(nxm), taking into account
C the vector of weights Dip(n). For each variable y of Y and each 
C variable x of X, the weighted multiple linear correlation R(y,x)  
C is computed between vectors y and (x and x^2). The sign of the simple 
C weighted linear correlation between y and x is assignedto R(y,x) 
C to obtain the final value CorPol(y,x). The matrix of weighted multiple
C linear correlations CorPol is used in polynomial CCA to represent the 
C explanatory variables X in the biplot diagram.              
      INCLUDE 'Rdparam.f'  
      Parameter (nmax1=nmax)
      Integer n,p,m,i,j,j1,m1,Variable01(nmax)
      Real*8 Z(nmax,nmax),X(nmax,nmax),CorPol(nmax,nmax),R12,R13,R23,D,
     +       X1(nmax1,nmax1),Z1(nmax1,nmax1),CorLinPond(nmax1,nmax1),
     +       X2(nmax1,nmax1),Cor(nmax1,nmax1),D22,Dip(nmax)
       
       call CorrelationPonderee(Z,X,Dip,CorLinPond,n,p,m)        
       do 5 i=1,m     
       do 5 j=1,p 
     
       do 10 i1=1,n
       X1(i1,1)=X(i1,i)
       X2(i1,1)=X(i1,i)**2
   10  Z1(i1,1)=Z(i1,j)
    
       call CorrelationPonderee(Z1,X1,Dip,Cor,n,1,1)
       R12=Cor(1,1)        
       
       if (Variable01(i).eq.0) then
          call CorrelationPonderee(Z1,X2,Dip,Cor,n,1,1)
          R13=Cor(1,1)
          call CorrelationPonderee(X1,X2,Dip,Cor,n,1,1)
          R23=Cor(1,1)
          D22=1-R23**2
          D=1-R23*R23-R12*R12+R12*R23*R13-R13*R13+R13*R12*R23
          CorPol(i,j)=sqrt(1-D/D22)
          CorPol(i,j)=dsign(CorPol(i,j),CorLinPond(i,j))
       else 
          CorPol(i,j)=R12
       end if   
         
    5  continue
       return
       end
      
      
      Subroutine CorrelationPonderee(Z,X,Dip,Cor,n,nvalc,m) 
C Calcul de la matrice des coefficients de correlation ponderes 
C Cor(m,nvalc) entre les matrices Z(nxnvalc) et X(nxm), 
C tenant compte du vecteur des poids Dip(n).
C
C Calculation of the matrix of weighted correlation coefficients 
C Cor(m,nvalc) between matrices Z(nxnvalc) and X(nxm),  
C taking into account the vectorof weights Dip(n).
      INCLUDE 'Rdparam.f'       
      Parameter (nmax1=nmax)
      Integer n,nvalc,m,i,j,k
      Real*8 Z(nmax,nmax),X(nmax,nmax),Cor(nmax,nmax),Dip(nmax1),ak,Dpp
            
      Dpp=0.0 
      do 3 i=1,n
    3 Dpp=Dpp+Dip(i)
    
      do 10 k=1,nvalc
      do 10 j=1,m
      sx=0.0
      sx2=0.0
      sy=0.0
      sy2=0.0
      sxy=0.0
      do 20 i=1,n
      sx=sx+Dip(i)*X(i,j)
      sx2=sx2+Dip(i)*(X(i,j)**2)
      sy=sy+Dip(i)*Z(i,k)
      sy2=sy2+Dip(i)*(Z(i,k)**2)
  20  sxy=sxy+Dip(i)*X(i,j)*Z(i,k)
      
      Cor(j,k)=(sxy-(sx*sy/Dpp))/dsqrt((sx2-(sx**2/Dpp))*
     +         (sy2-(sy**2/Dpp)))
  10  continue
      return
      end
      
      
      Subroutine Eigen(n,p,nmax,langue,irdacca,ican,nombre,
     + nvalpr,Yhat,totinert,Lambda,SingVal,U,UL,E,Z,vect,LUT,temp)
      
      Integer p,nombre(nmax)
      Real*8 Yhat(nmax,nmax),totinert
      Real*8 Lambda(nmax),SingVal(nmax),U(nmax,nmax),UL(nmax,nmax)
      Real*8 Z(nmax,nmax),vect(nmax),temp,dflnm1,E(nmax)
      dflnm1=dfloat(n-1)
      limite=min(n,p)
C Householder: calcul de la matrice Yhat'Yhat,
C puis des valeurs et vecteurs propres 
C Compute SSCP matrix (Yhat'Yhat),
C followed by eigenvalues and eigenvectors
      do 2 i=1,n
      do 2 j=1,p
    2 Z(j,i)=Yhat(i,j)
      if(n.ge.p) then
         call Produit(Z,Yhat,U,p,n,p,nmax)
         call tred2(U,p,nmax,Lambda,E)
         call tqli(Lambda,E,p,nmax,U)
C Householder pour matrices ou n < p 
C Householder for matrices where n < p
         else
   80    call Produit(Yhat,Z,U,n,p,n,nmax)
         call tred2(U,n,nmax,Lambda,E)
         call tqli(Lambda,E,n,nmax,U)
         do 4 i=1,n
    4    SingVal(i)=dsqrt(Lambda(i))
         endif
C Transformation de U lorsque n < p (eq. 9.39)
C Transformation of U when n < p (eq. 9.39)
      if(n.lt.p) then
C        U = Yhat' * Eigenvectors * (Singular values)-1
C        Z contient Yhat'. UL est une matrice intermediaire de calcul.
C        Z contains Yhat'. UL is used here as a scratch file.
         do 22 i=1,n
         do 22 j=1,n
   22    UL(i,j)=U(i,j)/SingVal(j)
         call Produit(Z,UL,U,p,n,n,nmax)
         endif
C
C Les valeurs propres sont dans Lambda;
C les vecteurs propres normalises sont dans U.
C The eigenvalues are in vector Lambda;
C the normalized eigenvectors are in U.
C
C Tri des valeurs et vecteurs propres en ordre decroissant
C des valeurs propres.
C Put the eigenvalues and corresponding eigenvectors in
C descending order.
C Phase 1: Trier les valeurs propres ainsi que le vecteur
C d'entiers "nombre"
C Phase 1: Sort eigenvalues together with the vector
C of integers "nombre"
         do 30 i=1,limite
   30    nombre(i)=i
         do 36 j=2,limite
            temp=Lambda(j)
            nn=nombre(j)
            do 32 i=j-1,1,-1
               if(Lambda(i).gt.temp) goto 34
               Lambda(i+1)=Lambda(i)
               nombre(i+1)=nombre(i)
   32          continue
            i=0
   34       Lambda(i+1)=temp
            nombre(i+1)=nn
   36    continue
C Phase 2: Trier U selon le nouvel ordre donne par "nombre";
C "vect" est un tampon.
C Phase 2: Sort U according to new order of "nombre".
C Use "vect" as buffer.
         do 42 jj=1,p
         do 38 j=1,limite
   38    vect(j)=U(jj,j)
         do 40 j=1,limite
   40    U(jj,j)=vect(nombre(j))
   42    continue
C Phase 3: eliminer les valeurs propres nulles
C Phase 3: eliminate null eigenvalues
      nval=0
      do 44 i=1,limite
      if(Lambda(i).gt.0.000001) then
         nval=nval+1
         else
         goto 46
         endif
   44 continue
   46 if(nval.lt.nvalpr) nvalpr=nval
      if(nval.eq.0) then
         temp=0.0
         goto 56
         endif
C Impression des valeurs propres / Print eigenvalues
      if(ican.eq.1) then
         if(langue.eq.1) write(LUT,101)
         if(langue.eq.2) write(LUT,201)
         else
         if(langue.eq.1) write(LUT,102)
         if(langue.eq.2) write(LUT,202)
         endif
      if(irdacca.eq.1) then
         do 48 j=1,nvalpr
         SingVal(j)=dsqrt(Lambda(j)/dflnm1)
   48    Lambda(j)=Lambda(j)/dflnm1
         else
         do 50 j=1,nvalpr
   50    SingVal(j)=dsqrt(Lambda(j))
         endif
      write(LUT,100) (Lambda(i), i=1,nvalpr)
      if(irdacca.eq.1) then
         if(langue.eq.1) write(LUT,108) 
         if(langue.eq.2) write(LUT,208)
         else
         if(langue.eq.1) write(LUT,103) 
         if(langue.eq.2) write(LUT,203)
         endif
      write(LUT,100) (100.0*Lambda(i)/totinert, i=1,nvalpr)
      temp=0.0
      do 52 i=1,nvalpr
      temp=temp+Lambda(i)
   52 vect(i)=temp
      if(irdacca.eq.1) then
         if(langue.eq.1) write(LUT,109) 
         if(langue.eq.2) write(LUT,209)
         else
         if(langue.eq.1) write(LUT,104)
         if(langue.eq.2) write(LUT,204)
         endif
      write(LUT,100) (100.0*vect(i)/totinert, i=1,nvalpr)
      temp=0.0
      do 54 i=1,nvalpr
   54 temp=temp+Lambda(i)
      if(ican.eq.1) then
         if(langue.eq.1) write(LUT,105)
         if(langue.eq.2) write(LUT,205)
         write(LUT,100) (100.0*vect(i)/temp, i=1,nvalpr)
         endif
   56 if(ican.eq.1) then
         if(langue.eq.1) write(LUT,106) temp
         if(langue.eq.2) write(LUT,206) temp
         else
         if(langue.eq.1) write(LUT,107) temp
         if(langue.eq.2) write(LUT,207) temp
         endif
         write(LUT,*)
      return
  100 format(8F10.5)
  101 format(/' Valeurs propres canoniques')
  201 format(/' Canonical eigenvalues')
  102 format(/' Valeurs propres non canoniques')
  202 format(/' Non-canonical eigenvalues')
  103 format(/' % de la variance totale de l''AFC')
  203 format(/' % of total variance of CA')
  104 format(/' % cumulatif de la variance totale de l''AFC')
  204 format(/' Cumulative % of total variance of CA')
  105 format(/' % cumulatif de la variance canonique')
  205 format(/' Cumulative % of canonical variance')
  106 format(/' Somme des valeurs propres canoniques',F15.5)
  206 format(/' Sum of all canonical eigenvalues',F15.5)
  107 format(/' Somme des valeurs propres non canoniques',F15.5)
  207 format(/' Sum of non-canonical eigenvalues',F15.5)
  108 format(/' % de la variance totale de l''ACP')
  208 format(/' % of total variance of PCA')
  109 format(/' % cumulatif de la variance totale de l''ACP')
  209 format(/' Cumulative % of total variance of PCA')
      end


      Subroutine Inertie(Y,Y1,inertie1,inertie2,n,p,irdacca)
C Calcul de l'inertie totale de Y et Y1
C Compute total inertia in Y and Y1       
      INCLUDE 'Rdparam.f'
      Parameter (nmax1=nmax)
      Integer n,p,m,i,j,irdacca           
      Real*8 Y(nmax,nmax),Y1(nmax,nmax),sx(nmax1),sx1(nmax1),inertie1,   
     +       Ycentre(nmax1,nmax1),Y1centre(nmax1,nmax1),inertie2
      
C Calcul de l'inertie totale de Y et Y1 en ACC
C Compute total inertia in Y and Y1 in CCA
      
      if (irdacca.eq.2) then      
      inertie1=0.0
      inertie2=0.0
      do 16 i=1,n
      do 16 j=1,p
      inertie1=inertie1+Y(i,j)**2
      inertie2=inertie2+Y1(i,j)**2
   16 continue      
            
      else

C Calcul de l'inertie totale de Y et Y1 en ACR
C Compute total inertia in Y and Y1 in RDA 
          
      dfln=dfloat(n)
      dflnm1=dfloat(n-1)
      do 8 j=1,p
      sx(j)=0.0
    8 sx1(j)=0.0
      do 10 i=1,n
      do 10 j=1,p
      sx(j)=sx(j)+Y(i,j)
   10 sx1(j)=sx1(j)+Y1(i,j)
      do 12 j=1,p
      sx(j)=sx(j)/dfln
   12 sx1(j)=sx1(j)/dfln
      do 14 i=1,n
      do 14 j=1,p
      Ycentre(i,j)=Y(i,j)-sx(j)
   14 Y1centre(i,j)=Y1(i,j)-sx1(j)
   
      inertie1=0.0
      inertie2=0.0
      do 18 i=1,n
      do 18 j=1,p
      inertie1=inertie1+Ycentre(i,j)**2
      inertie2=inertie2+Y1centre(i,j)**2
   18 continue
      end if
      
      return
      end
     

      Subroutine Linreg(X,Y,Yhat,B,n,p,m)
C Procedure de regression lineaire multiple de la matrice Y sur la 
C matrice X. Si la matrice X est singuliere et la valeur
C Yhat(nmax,nmax) a l'entree de la procedure est 999, alors les colonnes
C de X qui sont lineairement dependantes sont supprimees de X.
C
C Procedure for multiple linear regression of matrix Y on matrix X.
C If X is singular and the input value Yhat(nmax,nmax) is 999,
C the linearly dependent columns of X are removed from X. 
      INCLUDE 'Rdparam.f'
      Parameter (nmax1=nmax)
      Integer n,m,p,i,j,k,k1,m1
      Real*8 X(nmax,nmax),Y(nmax,nmax),Yhat(nmax,nmax),B(nmax,nmax),
     +       Xprime(nmax1,nmax1),Inverse(nmax1,nmax1),Xred(nmax1,nmax1)

      do 26 i=1,n
      do 26 j=1,m
  26  Xprime(j,i)=X(i,j)   
      call Produit(Xprime,X,Inverse,m,n,m,nmax)
      a=GaussJ1(Inverse,m)
              
      if (a.eq.1) then
C elimination des variables dependantes de la matrice X,
C suivie du calcul de la matrice des valeurs ajustees Yhat.
C Elmination procedure for the linearly dependent variables of X,
C followed by calculation of the matrix of fitted values Yhat.
         k1=1
         k=2
         m1=m
         k2=1                  
  10     continue
         if (B(nmax,nmax).eq.999) B(k2,nmax)=0
         do 12 i=1,n
         if (X(i,k2).ne.0.) go to 11
  12     continue 
         if (k2.eq.m) then
             pause 'All matrix values are 0'
         else
             k2=k2+1
             go to 10
         end if             
  11     continue
         if (B(nmax,nmax).eq.999) B(k2,nmax)=1
  
         do 14 i=1,n
         Xred(i,1)=X(i,1)
  14     Xprime(1,i)=X(i,1)
         if (k2.eq.m) then
            k=1 
            go to 20     
         end if       
                 
         do while (k1.le.m) 
           do 15 i=1,n
           Xred(i,k)=X(i,k1)
  15       Xprime(k,i)=X(i,k1)
                 
           call Produit(Xprime,Xred,Inverse,k,n,k,nmax)
           a=GaussJ1(Inverse,k)
           
           if (B(nmax,nmax).eq.999) then
              if (a.eq.0) then 
                 B(k1,nmax)=1
              else 
                 B(k1,nmax)=0
              end if
           end if   

           if ((a.eq.0).and.(k1.ne.m)) k=k+1
           if ((a.eq.1).and.(k1.eq.m)) k=k-1
           
           k1=k1+1           
         end do
         
  20     call Produit(Xprime,Xred,Inverse,k,n,k,nmax)
         a=GaussJ1(Inverse,k) 
         
         call Produit(Inverse,Xprime,Yhat,k,k,n,nmax)      
         call Produit(Yhat,Y,B,k,n,p,nmax)      
         call Produit(Xred,B,Yhat,n,k,p,nmax)
         
C eliminer les variables colineaires de X si Yhat(nmax,nmax)=999
C Elimination of collinear variables in X if Yhat(nmax,nmax)=999                   
         if (Yhat(nmax,nmax).eq.999) then        
            m=k
            do 25 i=1,n
            do 25 j=1,k
  25        X(i,j)=Xred(i,j)  
         end if
         go to 100
        
      end if  

C Calcul de la matrice des valeurs ajustees Yhat si la matrice X
C a l'entree du sous-programme n'est pas singuliere.
C Compute the matrix of fitted values Yhat if the input matrix X 
C is not singular.
      call Produit(Inverse,Xprime,Yhat,m,m,n,nmax)      
      call Produit(Yhat,Y,B,m,n,p,nmax)      
      call Produit(X,B,Yhat,n,m,p,nmax)      
 100  continue
C
      return
      end


      Subroutine PermutationTest(YY,XX,Fmoyen,n,p,m,Dip,irdacca,
     +                           Permnumber,Variable01,langue,LUT)  

C Procedure de test par permutation pour les methodes de regression 
C lineaire et polynomiale. La procedure permet de trouver la 
C signification des deux modeles (lineaire et polynomial) en calculant
C la valeur de la probabilite P que chacun des deux modeles  
C n'explique pas les donnees examinees mieux que des associations 
C aleatoires entre Y et X; elle correspond a l'hypothese d'independance 
C (lineaire ou polynomiale) entre les matrices YY(nxp) et XX(nxm). Cette 
C procedure peut etre utilisee soit dans le cadre d'une ACR ou d'une 
C ACC. En ACC, le vecteur des poids Dip(n) est utilise dans le calcul.
C
C Le nombre de permutations est defini par le parametre Permnumber. 
C Le vecteur auxiliaire Variable01(m) identifie les variables binaires
C de la matrice X. Si la variable i (colonne i) de X est binaire, 
C alors Variable01(i)=1. 
C 
C Une procedure supplementaire de test par 
C permutation permet de verifier la signification de la quantite
C additionnelle de variance expliquee par le modele polynomial par 
C rapport au modele lineaire. La probabilite P trouvee par cette
C procedure est la probabilite que cette difference soit le
C fruit du hasard. Plus la valeur de P associee a la difference 
C de variance explique par les deux modeles est basse, meilleur  
C est le modele polynomial par rapport au modele lineaire. 
C
C Permutation testing procedure for linear and polynomial regression.
C This procedure allows the user to find the significance of the two 
C models (linear and polynomial). The probability P is that of the  
C hypothesis that each of the models does not explain the data better
C than random associations between Y and X would; it corresponds to 
C the hypothesis of independence (linear or polynomial) between 
C matrices YY(nxp) and XX(nxm). The procedure can be implemented 
C in either the RDA or CCA framework. In CCA, the vector of 
C weights Dip(n) is used in the calculation. 
C
C The number of permutations is given by the parameter Permnumber. 
C The auxiliary vector Variable01(m) identifies the binary variables
C in X. If variable i (column i) of X is binary, then Variable01(i)=1. 
C
C A supplementary permutation testing procedure 
C allows one to check the significance of the 
C additional quantity of variance accounted for by the polynomial model,
C compared to the linear model. Probability P found by this procedure
C is the probability that the difference in variance explained by the 
C two models is the result of random association. The smaller the value 
C of P corresponding to the difference in variance accounted for,
C the better the polynomial model is.
            
      INCLUDE 'Rdparam.f'
      Parameter (nmax1=nmax)
      Integer n,p,m,i,j,j1,m1,i1,i2,regtype,degre,Flag(nmax1),
     + a,irdacca,iordre(nmax1),iseed,Permnumber,iii,
     + Model,Impression,deglim(nmax1),
     + Variable01(nmax),Variable01Copy(nmax1),Variable01cour(nmax1)
            
      Real*8 YY(nmax,nmax),XX(nmax,nmax),Y(nmax1,nmax1),
     + X(nmax1,nmax1),S,S1,S2,Fmoyen(2),Z(nmax1,nmax1),Dip(nmax),
     + Xtr(nmax1,nmax1),Yhat(nmax1,nmax1),B(nmax1,nmax1),
     + ZZ(nmax1,nmax1),Yres(nmax1,nmax1),Xtran(nmax1,nmax1),
     + Yhatiim(nmax1,nmax1),Yhatres(nmax1,nmax1),
     + Yhati(nmax1,nmax1),Yhatii(nmax1,nmax1),B1(nmax1,nmax1),
     + inertie1,inertie2,SommeValCanRes,FmoyenRes,Xaux(nmax1,nmax1),
     + Xres(nmax1,nmax1),ZD(nmax1,nmax1),ZD1(nmax1,nmax1),
     + Aux(nmax1,nmax1),VAR_pol,VAR_lin,VAR_Y
     
  2   if (langue.eq.1) then
         write(*,120)     
      else
         write(*,220)   
      end if
      read(*,*) Model      
      if ((Model.ne.1).and.(Model.ne.2).and.(Model.ne.3)) go to 2

  3   if (langue.eq.1) then
         write(*,110)     
      else
         write(*,210)   
      end if
      read(*,*) Impression      
      if ((Impression.ne.1).and.(Impression.ne.2)) go to 3
      
      if (langue.eq.1) then        
         write(LUT,*)'Pourcentage de variance associe a',
     +               ' la matrice Y (non permutee)' 
      else
         write(LUT,*)'Percentage of variance associated',
     +               ' with matrix Y (unpermuted)'             
      end if
      write(LUT,*)
         
      a=1
      j1=0
      j2=0
      if (Model.ne.2) Fmoyen(1)=1.0/Permnumber 
      if (Model.ne.1) Fmoyen(2)=1.0/Permnumber      
      if (Model.eq.3) FmoyenRes=1.0/Permnumber       
      iseed=0
      do 13 i=1,100
   13 turn=rand(iseed)
      
C elimination des variables colineaires de la matrice XX(matrice X)
C Elimination of collinear variables from matrix XX(matrix X)

      if (Model.ne.1) then
      do 5 i=1,m
      Variable01Cour(i)=Variable01(i)
  5   Variable01Copy(i)=Variable01(i)
           
      Variable01(m)=Variable01(1)
      Variable01Cour(m)=Variable01(1)
      Variable01(1)=1
      Variable01Cour(1)=1
      end if
      
      do 4 i=1,n
      iordre(i)=i
      
      do 7 j=1,p
   7  Y(i,j)=YY(i,j)
      
      do 8 j=2,m-1        
      X(i,j)=XX(i,j)        
   8  continue
      X(i,1)=XX(i,m)
      X(i,m)=XX(i,1)     
   4  continue  
     
      Yhat(nmax,nmax)=999
      B(nmax,nmax)=999
      mm1=m  
      
      call Linreg(X,Y,Yhat,B,n,p,m)
      Yhat(nmax,nmax)=0.0
      B(nmax,nmax)=0.0      

C Mettre a jour les variables binaires si la matrice XX est singuliere 
C Update the binary variables if XX is singular     
      if (Model.ne.1) then
      if (mm1.gt.m) then
         i=2
         i1=2
         do while (i1.lt.m+1)      
            if (B(i,nmax).eq.0) then
               i=i+1      
               do while (B(i,nmax).eq.0)
                  i=i+1
               end do
            end if   
            Variable01Cour(i1)=Variable01(i)
            i=i+1
            i1=i1+1
         end do     
      end if
      end if 
C Reduction du temps de calcul en ACR
C (calcul de la matrice Aux=X*[X'*X]^-1*X')
C Reduction of computation time for RDA
C (calculation of matrix Aux=X*[X'*X]^-1*X')     
      if ((irdacca.eq.1).and.(model.ne.2)) then
          do 26 i=1,n
          do 26 j=1,m
  26         Xtr(j,i)=X(i,j)   
          call Produit(Xtr,X,ZZ,m,n,m,nmax)
          call GaussJ(ZZ,m)   
          call Produit(ZZ,Xtr,ZD,m,m,n,nmax) 
          call Produit(X,ZD,Aux,n,m,n,nmax)                         
      end if 
                    
     
      go to 16
                 
C Cycle des permutations
C Permutation loop      
      
   9  a=0      
      write(*,*)
      write(LUT,*)
      if (langue.eq.1) then
         write(*,*) 'Debut des tests par permutation (',(Permnumber-1),
     +   ' permutations)'
         if (LUT.eq.7) write(LUT,108) (Permnumber-1)
      else
         write(*,*) 'Permutation tests started (',(Permnumber-1),
     +   ' permutations) ...'
         if (LUT.eq.7) write(LUT,208) (Permnumber-1)
      end if
      write(LUT,*)
      do 10 iperm=1,Permnumber-1  
      
      call Permute (iordre,iseed,n,nmax)
             
      do 14 i=1,n
      do 14 j=1,p      
  14  Y(i,j)=YY(iordre(i),j)
             
  16  continue
       
C Regression lineaire
C Linear regression
 
      if (model.ne.2) then            
            if (irdacca.eq.1) then
               call produit(Aux,Y,Yhat,n,n,p,nmax)             
            else 
               do 21 i=1,n
               do 21 j=1,m      
  21              ZD(i,j)=X(i,j)*dsqrt(Dip(iordre(i)))             
               call Linreg(ZD,Y,Yhat,B,n,p,m)
            end if 
                                 
C Calcul de la statistique pseudo-F
C Compute the pseudo-F statistic
      call Inertie(Y,Yhat,inertie1,inertie2,n,p,irdacca)
      VAR_lin=inertie2/inertie1
      if (a.eq.1) SommeValCan1=(inertie2/inertie1)
      
      if (a.eq.0) then
          if (SommeValCan1.le.(inertie2/inertie1))
     +        Fmoyen(1)=Fmoyen(1)+1.0/Permnumber      
      end if
       if ((a.eq.1).or.(Impression.eq.1)) then
          if (langue.eq.1) then
              write(LUT,102) 100*(inertie2/inertie1)
          else 
              write(LUT,202) 100*(inertie2/inertie1)
          end if
       end if    
      end if

C Regression polynomiale
C Polynomial regression

      if (model.ne.1) then   
             regtype=7
             Yhatii(nmax,nmax)=999      
	      do 500 k=1,p
C Limiter le degre du polynome
C Limit the degree of the polynomial	      
	      do 501 i=1,m
	         Variable01(i)=Variable01Cour(i)    
  501	         deglim(i)=1    

C Assembler la matrice Z qui est la matrice X (Xreduite),
C avec la colonne de 1 placee en premiere position.
C Composer le vecteur ZZ qui est le vecteur numero k de la matrice Y.
C Construct matrix Z (Xreduced) where the column of 1 is placed
C in first position. Vector ZZ contains vector number k of matrix Y.
             do 590 i=1,n
	         ZZ(i,1)=Y(i,k)
	      do 590 j=1,m
  590	         Z(i,j)=X(i,j)
	      
C Boucle de la regression polynomiale pour la colonne k de Y
C Loop of polynomial regression for column k of Y
             do 520 k1=m,3,-1
                          
             if (irdacca.eq.2) then
                do 521 i=1,n
                do 521 j=1,m
  521           ZD(i,j)=Z(i,j)*dsqrt(Dip(iordre(i))) 
                call Linreg (ZD,ZZ,Yhati,B,n,1,k1)
             else
                call Linreg (Z,ZZ,Yhati,B,n,1,k1)	      
             end if             
           
	      do 530 i=2,k1
	      do 530 j=i+1,k1
	      
C Integration des colonnes i et j de la matrice Z (Xreduite)
C Integration of columns i and j of matrix Z (Xreduced) 	            
	      do 540 i1=1,n
	      
	      Yres(i1,1)=ZZ(i1,1)-Yhati(i1,1)
	      Xtran(i1,1)=Z(i1,i)
	      Xtran(i1,2)=Z(i1,j)
	      Xtran(i1,3)=Z(i1,i)*Z(i1,j)
             ii=4           
                   
	      if ((deglim(i).ne.0).and.(Variable01(i).ne.1)) then	          
	          Xtran(i1,4)=Z(i1,i)**2
	          ii=5              	          
	      end if   
	        	      
	      if ((deglim(j).ne.0).and.(Variable01(j).ne.1)) then	          
	          Xtran(i1,ii)=Z(i1,j)**2
	          ii=ii+1	          	          	          
	      end if
	      
  540         continue
             
C  eviter la matice singuliere: eliminer les variables 
C  constantes dans Xtran
C  Avoid a singular matrix: eliminate constant variables in Xtran	

              ii=ii-1
              do 543 j1=1,ii                 
  543            Flag(j1)=1
              
              do 545 j1=1,ii
              do 545 i1=1,n
                 Xaux(i1,j1)=Xtran(i1,j1)
                 if (Xtran(i1,j1).NE.Xtran(1,j1)) Flag(j1)=0                 
  545         continue 
             
             iii=1
             do 546 j1=1,ii
                if (Flag(j1).eq.0) then
                   do 547 i1=1,n                   
  547                 Xtran(i1,iii)=Xaux(i1,j1)
                     iii=iii+1
                end if
  546        continue
                                      
             do 548 i1=1,n
  548           Xtran(i1,iii)=Z(i1,1)

C  Regression polynomiale; mise a jour de la meilleure
C  paire de colonnes i,j
C  Polynomial regressing; update the best pair of columns i and j             
            
            if (irdacca.eq.2) then
                do 522 i1=1,n
                do 522 j1=1,iii      
  522           ZD(i1,j1)=Xtran(i1,j1)*dsqrt(Dip(iordre(i1))) 
                call Linreg (ZD,Yres,Yhatii,B1,n,1,iii)
             else
                call Linreg (Xtran,Yres,Yhatii,B1,n,1,iii) 
             end if         
                                     
                            
	      S=0.0
             do 550 i1=1,n
  550	      S=S+(ZZ(i1,1)-Yhati(i1,1)-Yhatii(i1,1))**2
             
             if (((i.eq.2).and.(j.eq.3)).or.(S.lt.Smin)) then
	      Smin=S
	      im=i
	      jm=j
	      do 580 i1=1,n
  580	      Yhatiim(i1,1)=Yhatii(i1,1)
              end if
  530	      continue

C Reduire la matrice Z (Xreduite); mise a jour du vecteur
C de limitation 'deglim'
C Reduce matrix Z(Xreduced); update the vector of limitation 'deglim'
	      do 560 i1=1,n
	      if (irdacca.eq.2) then
 	        Z(i1,im)=Z(i1,im)*B(im,1)+Z(i1,jm)*
     +                  B(jm,1)+Yhatiim(i1,1)/dsqrt(Dip(iordre(i1)))
             else 
               Z(i1,im)=Z(i1,im)*B(im,1)+Z(i1,jm)*B(jm,1)+Yhatiim(i1,1)
             end if    
  560        continue
                          
             deglim(im)=0
              Variable01(im)=0             
              if (k1.gt.jm) then
                  deglim(jm)=deglim(k1)
                  Variable01(jm)=Variable01(k1)           
	          do 570 i1=1,n
  570               Z(i1,jm)=Z(i1,k1)
              end if	     	     	            	      
  520	      continue
                
C Calculer la colonne k de la matrice Yhat
C Compute column k of matrix Yhat
              if (irdacca.eq.2) then
                do 523 i=1,n
                do 523 j=1,2
  523           ZD(i,j)=Z(i,j)*dsqrt(Dip(iordre(i))) 
                call Linreg (ZD,ZZ,Yhati,B,n,1,2) 
             else
                call Linreg (Z,ZZ,Yhati,B,n,1,2)       
             end if
                   
              do 600 i=1,n
  600         Yhat(i,k)=Yhati(i,1)  
  500         continue
  
C Calculer la statistique pseudo-F pour la regression polynomiale
C Compute the pseudo-F statistic for the polynomial regression
  
      call Inertie(Y,Yhat,inertie1,inertie2,n,p,irdacca)
      VAR_pol=inertie2/inertie1
      if (a.eq.1) SommeValCan2=(inertie2/inertie1)      
      
      if (a.eq.0) then
          if (SommeValCan2.le.(inertie2/inertie1))
     +        Fmoyen(2)=Fmoyen(2)+1.0/Permnumber        
      end if
       
      if ((a.eq.1).or.(Impression.eq.1)) then
          if (langue.eq.1) then
              write(LUT,106) 100*(inertie2/inertie1)
          else 
              write(LUT,206) 100*(inertie2/inertie1)
          end if  
       end if      
      
CAnalyse de la difference de variance expliquee par les modeles lineaire
C et polynomial. Calcul de la statistique pseudo-F pour la difference.
C Analysis of the difference of variance accounted for by the polynomal
C model, compared to the linear model. Compute the pseudo-F statistic
C for the difference.
      if (Model.eq.3) then
          VAR_Y=1.0      
          if (a.eq.1) SommeValCanRes=(VAR_pol-VAR_lin)/(VAR_Y-VAR_pol)        
         
         if (a.eq.0) then                      
           if (SommeValCanRes.le.(VAR_pol-VAR_lin)/(VAR_Y-VAR_pol))
     +         FmoyenRes=FmoyenRes+1.0/Permnumber
         end if
     
         if ((a.eq.1).or.(Impression.eq.1)) then
          if (langue.eq.1) then
               write(LUT,104) 100*(VAR_pol-VAR_lin)
          else 
               write(LUT,204) 100*(VAR_pol-VAR_lin)
          end if
          write(LUT,*) 
         end if        
      end if  
      
      end if 

      if (a.eq.1) go to 9
   
  10  continue 
       
C Mise a jour des variables modifiees et impression des valeurs
C de probabilite P
C Update the modified variables and print probability P     
      if (Model.ne.1) then
      do 135 i=2,m-1
 135  Variable01(i)=Variable01Copy(i)       
      Variable01(m)=Variable01Copy(1)      
      Variable01(1)=Variable01Copy(m)
      end if      
      m=mm1
            
      write(LUT,*)
      if (Model.ne.2) write(LUT,401) 'P(Lin)', Fmoyen(1)
      if (Model.ne.1) write(LUT,401) 'P(Pol)', Fmoyen(2)
      if (Model.eq.3) write(LUT,401) 'P(Pol-Lin)', FmoyenRes
      write(LUT,*)
      return
 401  format(1x,A10,F10.5)
 101  format(1x,2F10.5)
 102  format(' Pourcentage de variance',
     +       ' (regression lineaire) =    ', F10.5)
 202  format(' Percentage of variance',
     +       ' (linear regression) =        ', F10.5)
 104  format(' Difference de variance',
     +       ' expliquee (Pol-Lin) =       ', F10.5)
 204  format(' Difference of variance',
     +       ' accounted for (Pol-Lin) =    ', F10.5)
 106  format(' Pourcentage de variance',
     +       ' (regression polynomiale) = ', F10.5)
 206  format(' Percentage of variance',
     +       ' (polynomial regression) =    ', F10.5)
 108  format(' Debut des tests par permutation (',i5,' permutations)')
 208  format(' Permutation tests started (',i5,' permutations) ...')
 
 110  format(/' Imprimer le pourcentage de variance explique par le'
     +       /' modeles choisi a chaque iteration de la boucle des',
     +        ' permutations?'
     +       /'   (1) Imprimer le pourcentage de variance apres',
     +        ' chaque iteration'
     +       /'   (2) Ne pas imprimer le pourcentage de variance',
     +        ' apres chaque iteration')
          
 210  format(/' Print the percentage of variance accounted for'
     +       /' by the chosen model after each iteration of the',
     +        ' permutation loop?'
     +       /'   (1) Print the percentage of variance after each',
     +        ' iteration'
     +       /'   (2) Do not print the percentage of variance after',
     +        ' each iteration')  

 120  format(/' Tester la signification des modeles:'
     +       /'   (1) Lineaire seulement?'
     +       /'   (2) Polynomial seulement?'
     +       /'   (3) Lineaire et polynomial, ainsi que',
     +        ' la difference entre eux?')
 
 220  format(/' Test the significance of:'
     +       /'   (1) The linear model only?'
     +       /'   (2) The polynomial model only?'
     +       /'   (3) Both the linear and polynomial models,'
     +       /'       and also the difference between them?')               
      end


      Subroutine Permute (iordre,iseed,k,nmax)
C This subroutine permutes the first 'k' values of vector 
C 'iordre' at randomin an equiprobable way. This property has been  
C checked through intensivesimulations.
      Integer iseed,k,i,itemp,km1,m,iordre(nmax)         
      m=k
      km1=k-1
      do 10 i=1,km1
    8    j = 1 + rand(iseed)*m
         if(j.gt.m) goto 8
         itemp = iordre(m)
         iordre(m) = iordre(j)
         iordre(j) = itemp
         m=m-1
   10 continue
      return
      end
           
      
      Subroutine Produit(Z1,Z2,Z3,n1,n2,n3,nmax)
      Real*8 Z1(nmax,nmax),Z2(nmax,nmax),Z3(nmax,nmax),temp
C Z1 * Z2 = Z3
      do 8 i=1,n1
      do 8 j=1,n3
      temp=0.0
      do 6 k=1,n2
    6 temp=temp+Z1(i,k)*Z2(k,j)
    8 Z3(i,j)=temp
      return
      end


      Subroutine ReadData(n,p,m,nmax,Y,X,langue,LUT)
      Integer p,first
      Real*8 Y(nmax,nmax),X(nmax,nmax),ref
      Character*79 nom

C     Lecture du tableau de donnees / Read the data file
      if(langue.eq.1) write(*,*) 'Fichier de donnees?'
      if(langue.eq.2) write(*,*) 'Data file?'
      read(*,*) nom
      open(2,file=nom,status='old')
C      inquire(unit=2,name=nom)
      read(2,*) n,p,m
      if(langue.eq.1) write(LUT,103) nom,n,p,m
      if(langue.eq.2) write(LUT,203) nom,n,p,m
      if((n.gt.nmax).or.(p.gt.nmax)) then
         if(langue.eq.1) write(LUT,101) 
         if(langue.eq.2) write(LUT,201)
         stop
         endif
      if(m.gt.n-1) then
         if(langue.eq.1) write(LUT,102) 
         if(langue.eq.2) write(LUT,202)
         stop
         endif
      do 10 i=1,n
   10 read(2,*) (Y(i,j), j=1,p),(X(i,k), k=1,m)
       
      m=m+1
      do 11 i=1,n
   11 X(i,m)=1

C On previent l'utilisateur que les matrices X et Y on des variables
C qui ne varient pas, ou que Y contient une ligne de zeros.
C Warn the user that matrix X or Y has  
C variables that do not vary, or that there are rows of 0 in Y.
        first=0
        do 15 j=1,p
        ref=Y(1,j)
        do 14 i=2,n        
        if (ref.ne.Y(i,j)) go to 15
   14   continue
        first=first+1
        if(first.eq.1) then
           if(langue.eq.1) write(*,104) 
           if(langue.eq.2) write(*,204) 
           endif
        write(*,'(I4,$)') j
   15   continue
        if(first.gt.0) write(*,*)
C
        first=0
        ref=0.0
        do 35 i=1,n
        do 34 j=1,p        
        if (Y(i,j).ne.ref) go to 35
   34   continue
        first=first+1
        if(first.eq.1) then
           if(langue.eq.1) write(*,105) 
           if(langue.eq.2) write(*,205) 
           endif
        write(*,'(I4,$)') i 
   35   continue
        if(first.gt.0) write(*,*)
C
        first=0
        do 25 j=1,m-1
        ref=X(1,j)
        do 24 i=2,n        
        if (ref.ne.X(i,j)) go to 25
   24   continue
        first=first+1
        if(first.eq.1) then
           if(langue.eq.1) write(*,106) 
           if(langue.eq.2) write(*,206) 
           endif
        write(*,'(I4,$)') j
   25   continue
        if(first.gt.0) write(*,*)
        
      return

C***********************************************************************
  100 format(8F10.5)
  101 format(' Matrice Y trop grande.',
     +       ' Redimensionnez et recompilez le programme.')
  201 format(' Y matrix is too large. Redimension and recompile',
     +       ' the program.')
  102 format(' La matrice X contient trop de variables explicatives',
     +       ' (m > n-1).')
  202 format(' The X matrix contains too many explanatory variables',
     +       ' (m > n-1).')
  103 format(/' Fichier de donnees: ',a30/
     +        i6,' sites'/
     +        i6,' variables reponse (matrice Y) '/
     +        i6,' variables explicatives (matrice X)')
  203 format(/' Input file: ',a30/
     +        i6,' sites'/
     +        i6,' response variables (matrix Y)'/
     +        i6,' explanatory variables (matrix X)')
  104 format(/' Attention: Les variables (colonnes) suivantes de la',
     +        ' matrice Y sont constantes:')
  204 format(/' Warning: The following variables (columns) of matrix Y',
     +       ' are constant:')
  105 format(/' Attention: Les lignes suivantes de la matrice Y ne',
     +        ' contiennent que des zeros   (ACC est impossible):')
  205 format(/' Warning: The following rows of matrix Y only contain',
     +        ' zeros (CCA is impossible):')
  106 format(/' Attention: Les variables (colonnes) suivantes de la',
     +        ' matrice X sont constantes:')
  206 format(/' Warning: The following variables (columns) of matrix X',
     +        ' are constant:')
      end


      Subroutine Regression(X,Y,Yhat,n,p,m,regtype,Variable01,irdacca,
     +                      Dip,LUT,langue)
      INCLUDE 'Rdparam.f'
      Parameter (nmax1=nmax)
      character*8 Char1
      Integer n,m,p,i,j,i1,j1,k,k1,k0,im,jm,regtype,principe,
     +       coeff,regtype1,m1,degre,Flag(6),iii,LUT,langue,
     +       NUM1(nmax1),Variable01(nmax),Variable01Copy(nmax1),
     +       COEFPRINT,COEFPRINT1,deglim(nmax),NUM(nmax1,nmax1),
     +       Variable01Cour(nmax1)
      Real*8 X(nmax,nmax),Y(nmax,nmax),Yhat(nmax,nmax),
     +       Yres(nmax1,nmax1),Rmin(nmax1),Copt(nmax1),E(nmax1,nmax1),
     +       Yhati(nmax1,nmax1),Z(nmax1,nmax1),ZZ(nmax1,nmax1),R,S,Sopt,
     +       B(nmax1,nmax1),B1(nmax1,nmax1),Xtran(nmax1,nmax1),
     +       Yhatii(nmax1,nmax1),Yhatiim(nmax1,nmax1),Smin,
     +       Ymoy(nmax1),Sopt1,S1,Sum,Xcopy(nmax1,nmax1),
     +	      Xaux(nmax1,nmax1),Ymoyhat(nmax1),Dip(nmax),ZD(nmax1,nmax1),
     +       Numero(nmax1),Coef(nmax1),NUMprel(nmax1)
     
C Choix du type de la regression/ Choice of type of regression      

      write(*,*)
   9  if(langue.eq.1) then
       write(*,*) 'Modele de regression multiple:'            
       write(*,*) '  (1) Regression lineaire Y = XB + C'      
       write(*,*) '  (2) Regression polynomiale Y = P(X)',
     +            '      (degre maximum des termes = 2)'
      else
       write(*,*) 'Multiple regression model:'            
       write(*,*) '  (1) Linear regression Y = XB + C'      
       write(*,*) '  (2) Polynomial regression Y = P(X)',
     +            '      (maximum degree for any term = 2)'      
      end if
            
      read(*,*) regtype
      if ((regtype.ne.1).and.(regtype.ne.2)) go to 9
 
C Mettre a jour la matrice X et eliminer les variables colineaires
C de la matrice X
C Update matrix X and remove collinear variables from X
           
      do 10 i=1,n
      do 10 j=1,m
 10   Xcopy(i,j)=X(i,j)
 
C Weighting of X prior to linear CCA
      if ((regtype.eq.1).and.(irdacca.eq.2)) then 
         do 28 i=1,n
         do 28 j=1,m
 28         X(i,j)=X(i,j)*dsqrt(Dip(i))
      end if
       
      do 14 i=1,m
      Variable01Cour(i)=Variable01(i)
 14   Variable01Copy(i)=Variable01(i)
           
      Variable01(m)=Variable01(1)
      Variable01Cour(m)=Variable01(1)
      Variable01(1)=1
      Variable01Cour(1)=1
      
      do 12 i=1,n
      S=X(i,m)
      X(i,m)=X(i,1)
 12   X(i,1)=S
      
      Yhat(nmax,nmax)=999
      B(nmax,nmax)=999
      mm1=m  
      
      call Linreg(X,Y,Yhat,B,n,p,m)
      Yhat(nmax,nmax)=0.0
      B(nmax,nmax)=0.0      
      Fl=0
      
      if (mm1.gt.m) then
         Fl=1 
         i=2
         i1=2
         do while (i1.lt.m+1)      
            if (B(i,nmax).eq.0) then
               i=i+1      
               do while (B(i,nmax).eq.0)
                  i=i+1
               end do
            end if   
            Variable01Cour(i1)=Variable01(i)
            NUMprel(i1)=i
            if (i.eq.mm1) NUMprel(i1)=1
            i=i+1
            i1=i1+1
         end do    
      end if        
      
      select case (regtype)           
           case (1)               
C Print the linear regression cofficients      
  95        if(langue.eq.1) then
             write(*,*)
             write(*,*) 'Impression:'
             write(*,*) '  (1) Imprimer les coefficients de',
     +                   ' regression lineaire'
             write(*,*) '  (2) Ne pas imprimer les coefficients',
     +                   ' de regression lineaire'
            else 
             write(*,*)
             write(*,*) 'Printing:'
             write(*,*) '  (1) Print the regression coefficients'
             write(*,*) '  (2) Do not print the regression coefficients'
            end if                                 
            read(*,*) COEFPRINT1
            if ((COEFPRINT1.ne.1).and.(COEFPRINT1.ne.2)) go to 95 
             
            if (COEFPRINT1.eq.1) then
                 if(langue.eq.1) then
                    write(LUT,*)
     +              '             Coefficients de regression lineaire'                    
                    write(LUT,109)        
                 else
                    write(LUT,*)
     +              '             Linear regression coefficients'                     
                    write(LUT,209)           
                 end if
                 write(LUT,*)      
             end if      
                  
            if (COEFPRINT1.eq.1) then                                     
                do 30 jj=1,p
                write(LUT,427) (B(j,jj), j=2,m)
                write(LUT,428)  B(1,jj)
  30            write(LUT,*)  
            end if
             
             
           case(2) 
C Print the polynomial regression cofficients  
  15         if(langue.eq.1) then
             write(*,*)
             write(*,*) 'Impression:'
             write(*,*) '  (1) Imprimer les coefficients des polynomes'
             write(*,*) '  (2) Ne pas imprimer les coefficients des',
     +                  ' polynomes'
             else 
             write(*,*)
             write(*,*) 'Printing:'
             write(*,*) '  (1) Print the polynomial coefficients'
             write(*,*) '  (2) Do not print the polynomial coefficients'
             end if
                                 
             read(*,*) COEFPRINT
             if ((COEFPRINT.ne.1).and.(COEFPRINT.ne.2)) go to 15
             
             if (COEFPRINT.eq.1) then
                 if(langue.eq.1) then
                 write(LUT,*) '             Coefficient des polynomes'                    
                 write(LUT,109)
                 write(LUT,110)
                 write(LUT,111) 
C                write(LUT,112)            
                 else
                 write(LUT,*) '             Polynomial coefficients'
                 write(LUT,209)
                 write(LUT,210) 
                 write(LUT,211) 
C                write(LUT,212)               
                 end if      
              end if      
                                          
             Yhatii(nmax,nmax)=999   
	      do 500 k=1,p
	      
C Introduire la limitation de degre des monomes
C Introduce the limitation of the degree of the monomials	      
	      do 501 i=1,m
	         Variable01(i)=Variable01Cour(i)
  501	         deglim(i)=1    

C  Composer la matrice Z (= Xreduite), avec la colonne
C  de 1 placee en premiere position.
C  Composer le vecteur ZZ qui est le vecteur numero k de la matrice Y.
C  Construct matrix Z (= matrix Xreduced) and vector ZZ
C  (vector k from matrix Y).
        
             do 591 i=1,n
                ZZ(i,1)=Y(i,k)
             do 591 j=1,m                     
  591           Z(i,j)=X(i,j)
 
             if (COEFPRINT.eq.1) then
             
             if(langue.eq.1) then
                write(LUT,120) k
             else
                write (LUT,220) k 
             end if 
               
                 do 515 j=2,m
                 if (Fl.eq.1) then
                    NUM(j,1)=NUMprel(j)
                 else
                    NUM(j,1)=j
                 end if
                 if (Fl.eq.0) NUM(m,1)=1                       
  515            NUM1(j)=1                                 
             end if
 
C Boucle de la regression polynomiale pour la colonne k de Y 
C Polynomial regression loop for column k of Y 
             do 520 k1=m,3,-1                                
                      
             if (irdacca.eq.2) then
                do 521 i=1,n
                do 521 j=1,m
  521           ZD(i,j)=Z(i,j)*dsqrt(Dip(i)) 
                call Linreg (ZD,ZZ,Yhati,B,n,1,k1)
             else
                call Linreg (Z,ZZ,Yhati,B,n,1,k1)	      
             end if
      
	      do 530 i=2,k1
	      do 530 j=i+1,k1
	      
C  Fusionner les colonnes i et j de la matrice Z (matrice Xreduite) 
C  Fuse columns i and j of matrix Z (matrix Xreduced)	            
	      do 540 i1=1,n
	      
	      Yres(i1,1)=ZZ(i1,1)-Yhati(i1,1)
	      Xtran(i1,1)=Z(i1,i)
	      Xtran(i1,2)=Z(i1,j)
	      Xtran(i1,3)=Z(i1,i)*Z(i1,j)
             ii=4           
                   
	      if ((deglim(i).ne.0).and.(Variable01(i).ne.1)) then	          
	          Xtran(i1,4)=Z(i1,i)**2
	          ii=5              	          
	      end if   
	        	      
	      if ((deglim(j).ne.0).and.(Variable01(j).ne.1)) then	          
	          Xtran(i1,ii)=Z(i1,j)**2
	          ii=ii+1	          	          	          
	      end if	      
                   
  540         continue
                
C  eviter la matice singuliere: eliminer les variables 
C  constantes dans Xtran
C  Avoid a singular matrix: eliminate constant variables in Xtran	
              ii=ii-1
              do 543 j1=1,ii                 
  543            Flag(j1)=1
              
              do 545 j1=1,ii
              do 545 i1=1,n
                 Xaux(i1,j1)=Xtran(i1,j1)
                 if (Xtran(i1,j1).NE.Xtran(1,j1)) Flag(j1)=0                 
  545         continue 
             
             iii=1
             do 546 j1=1,ii
                if (Flag(j1).eq.0) then
                   do 547 i1=1,n                   
  547                 Xtran(i1,iii)=Xaux(i1,j1)
                     iii=iii+1
                end if
  546        continue
            
             do 548 i1=1,n
  548           Xtran(i1,iii)=1.0
   
C  Regression polynomiale et mise a jour de la meilleure paire
C  de colonnes i,j
C  Polynomial regression procedure; update columns i and j
             if (irdacca.eq.2) then
                do 522 i1=1,n
                do 522 j1=1,iii                
  522           ZD(i1,j1)=Xtran(i1,j1)*dsqrt(Dip(i1)) 
                call Linreg (ZD,Yres,Yhatii,B1,n,1,iii)
             else
                call Linreg (Xtran,Yres,Yhatii,B1,n,1,iii) 
             end if         
          
	      S=0.0
             do 550 i1=1,n
  550	      S=S+(ZZ(i1,1)-Yhati(i1,1)-Yhatii(i1,1))**2
  
             if (((i.eq.2).and.(j.eq.3)).or.(S.lt.Smin)) then
	      Smin=S
	      im=i
	      jm=j
	      do 580 i1=1,n
  580	      Yhatiim(i1,1)=Yhatii(i1,1)
                          
              end if
  530	      continue
  
C Calcul des coefficients du monome courant
C Compute coefficients for the current monomial
C Nouvelle option: pour imprimer les coefficients de la CCA,
C  on multiplie Z par dsqrt(Dip(i1))
         if (COEFPRINT.eq.1) then   
	      	      
	      do 542 i1=1,6
  542	      Numero(i1)=0
	      	      
	      do 541 i1=1,n	      	    
	      Xtran(i1,1)=Z(i1,im)*dsqrt(Dip(i1))
	      Xtran(i1,2)=Z(i1,jm)*dsqrt(Dip(i1))
	      Xtran(i1,3)=Z(i1,im)*Z(i1,jm)*dsqrt(Dip(i1))
             ii=4           
                   
	      if ((deglim(im).ne.0).and.(Variable01(im).ne.1)) then	          
	          Xtran(i1,4)=(Z(i1,im)**2)*dsqrt(Dip(i1))
	          ii=5              	          
	      end if   
	        	      
	      if ((deglim(jm).ne.0).and.(Variable01(jm).ne.1)) then	          
	          Xtran(i1,ii)=(Z(i1,jm)**2)*dsqrt(Dip(i1))
	          ii=ii+1	          	          	          
	      end if
	      	      
             Xtran(i1,ii)=1.0*dsqrt(Dip(i1))     
  541        continue  

             B1(nmax,nmax)=999
             iii=ii           
             call Linreg (Xtran,Yres,Yhatii,B1,n,1,ii)
             B1(nmax,nmax)=0                 
       
             if (ii.eq.iii) then 
               Numero(1)=1
               Numero(2)=2
               Numero(3)=3
               iii=4
               if ((deglim(im).ne.0).and.(Variable01(im).ne.1)) then
                  Numero(4)=4
                  iii=iii+1
               end if
               if ((deglim(jm).ne.0).and.(Variable01(jm).ne.1)) then   
                  Numero(5)=iii
                  iii=iii+1
               end if                  
              Numero(6)=iii                            
             else
                i1=1
                i2=1
                do while (i1.lt.7)
                   if (B1(i1,nmax).eq.1.0) then 
                      Numero(i1)=i2
                      i2=i2+1   
                   end if
                i1=i1+1

                if ((i1.eq.4).and.((deglim(im).eq.0)
     +             .or.(Variable01(im).eq.1)))i1=i1+1                   
                if ((i1.eq.5).and.((deglim(jm).eq.0)
     +             .or.(Variable01(jm).eq.1)))i1=i1+1
                   
                end do    
              end if                   

	      if (Numero(1).ne.0) then
	         Coef(1)=B(im,1)+B1(Numero(1),1)
	      else
	         Coef(1)=B(im,1)
	      end if	      
	      if (Numero(2).ne.0) then
	         Coef(2)=B(jm,1)+B1(Numero(2),1)
	      else
	         Coef(2)=B(jm,1)
	      end if    	          
	      do 576 i1=3,6
	      if (Numero(i1).ne.0) then
	         Coef(i1)=B1(Numero(i1),1)
	      else
	         Coef(i1)=0   
	      end if	         	         
  576 	      continue 


             write(LUT,125)
             do 627 i1=1,NUM1(im)-1 
  627        write(LUT,'($,I3,$,A1,$)') NUM(im,i1),','
             write(LUT,127) NUM(im,NUM1(im))
             
             write(LUT,128)  
             do 628 i1=1,NUM1(jm)-1           
  628        write(LUT,'($,I3,$,A1,$)') NUM(jm,i1),','
             write(LUT,130) NUM(jm,NUM1(jm))

C             write(LUT,'(I3,$)') '(',(NUM(im,i1),',', i1=1,NUM1(im)-1),
C     +       NUM(im,NUM1(im)),')',' ','+',' ','(',(NUM(jm,i1),',',
C     +       i1=1,NUM1(jm)-1), NUM(jm,NUM1(jm)),')'                         	   

	      write(LUT,*)
	      write(LUT,199) (Coef(i1), i1=1,6)	
             
             do 552 i1=1,NUM1(jm)
  552        NUM(im,NUM1(im)+i1)=NUM(jm,i1)
             NUM1(im)=NUM1(im)+NUM1(jm)
             if (jm.ne.k1) then
                NUM1(jm)=NUM1(k1)
                do 553 i1=1,NUM1(k1)
  553           NUM(jm,i1)=NUM(jm,k1)	      
	      end if	      	   	
                  
          end if 

C Reduire la matrice Z (Xreduite) et mettre a jour le vecteur
C de limitation 'deglim'    
C Reduce matrix Z (Xreduced) and update the vector of degree
C limitation 'deglim'
	      do 560 i1=1,n
	      if (irdacca.eq.2) then
  	          Z(i1,im)=Z(i1,im)*B(im,1)+Z(i1,jm)*B(jm,1)+
     +                    Yhatiim(i1,1)/dsqrt(Dip(i1))
             else 
                 Z(i1,im)=Z(i1,im)*B(im,1)+Z(i1,jm)*B(jm,1)+
     +                    Yhatiim(i1,1)
             end if    
  560        continue
             
              deglim(im)=0 
              Variable01(im)=0
              
             if (k1.gt.jm) then
                 Variable01(jm)=Variable01(k1)
                  deglim(jm)=deglim(k1)
                 if (COEFPRINT.eq.1) then
                    NUM1(jm)=NUM1(k1)
                    do 569 i1=1,NUM1(k1)
  569                  NUM(jm,i1)=NUM(k1,i1)
	          end if 
	          
	             do 570 i1=1,n
  570                  Z(i1,jm)=Z(i1,k1)
             end if                   	     	     	            	      
  520	      continue
  
C Calculer la colonne k de la matrice Yhat
C Compute column k of matrix Yhat
             m1=2
             if (irdacca.eq.2) then
                do 523 i=1,n
                do 523 j=1,2
  523           ZD(i,j)=Z(i,j)*dsqrt(Dip(i)) 
                call Linreg (ZD,ZZ,Yhatii,B,n,1,m1) 
             else             
                call Linreg (Z,ZZ,Yhatii,B,n,1,m1)                  
             end if
        
C Option desactivee car les coefficients de la regression finale sont
C toujours 0 et 1
C              if (COEFPRINT.eq.1) then                             
C                 if (langue.eq.1) then     
C                     write(LUT,121)
C                 else 
C                     write(LUT,221)
C                 end if
C                                  
C                 if (m1.eq.2) then                   
C                    write(LUT,199) B(1,1), B(2,1)
C                 else 
C                    write(LUT,199) B(1,1)
C                 end if      
C              end if
                            
              do 600 i1=1,n
  600         Yhat(i1,k)=Yhatii(i1,1)  
  500         continue
  
      end select
        
      m=mm1
      
      do 145 j=1,m
      Variable01(j)=Variable01Copy(j)
      do 145 i=1,n      
  145 X(i,j)=Xcopy(i,j)
 
      do 160 j=1,p
      S=0.0
      S1=0.0
      do 161 i=1,n
      S1=S1+Yhat(i,j)
  161 S=S+Y(i,j)
      Ymoyhat(j)=S1/n
  160 Ymoy(j)=S/n
      
      
      S2=0.0   
      S=0.0
      Sqd=0.0
      Sopt=0.0
      Sum=0.0
      do 162 j=1,p
      S1=0.0
      Sopt1=0.0
        
      do 169 i=1,n      
      S1=S1+(Y(i,j)-Ymoy(j))**2
      Sopt1=Sopt1+(Yhat(i,j)-Ymoy(j))**2
      S2=S2+(Yhat(i,j)-Ymoyhat(j))**2      
 169  Sqd=Sqd+(Y(i,j)-Yhat(i,j))**2

      S=S+S1
      Sopt=Sopt+Sopt1
 162  Sum=Sum+Sopt1/S1
      
C Variance pour la CCA
C Variance in the case of CCA     
      if (irdacca.eq.2) then
         S1=0.0
         S2=0.0
         do 16 i=1,n
         do 16 j=1,p
         S1=S1+Y(i,j)**2
         S2=S2+Yhat(i,j)**2
   16    continue      
      end if

      write(LUT,*)
      if(langue.eq.1) then
              write(LUT,102) Sum/p
      else
              write(LUT,202) Sum/p      
      end if
      write(LUT,*)
      
      if (irdacca.eq.1) then
         if(langue.eq.1) write(LUT,106) 100*(1-Sqd/S)
         if(langue.eq.2) write(LUT,206) 100*(1-Sqd/S)
      else
         if(langue.eq.1) write(LUT,108) 100*S2/S1
         if(langue.eq.2) write(LUT,208) 100*S2/S1
      end if   
         
      write(LUT,*)
C      if(langue.eq.1) then
C          write(LUT,*) 'ecart quadratique moyen = ', Sqd/n/p
C      else    
C          write(LUT,*) 'Mean squared difference  = ', Sqd/n/p
C      end if    
C      write(LUT,*)
           
      return
 
C**********************************************************************
 100  format(8F10.5)
 199  format(6F12.6/)
 101  format(15i5)
 102  format(' Coefficient de determination moyen R^2  = ', F10.5) 
 202  format(' Mean coefficient of multiple determination',
     +       ' R^2  = ', F10.5) 
 
 104  format(2i1)
 105  format (3i3)
 
 106  format (' (RDA) Pourcentage de la variance totale de Y',
     +        ' explique =', F10.5)  
 206  format (' (RDA) Percentage of the total variance of Y',
     +        ' accounted for =', F10.5)
 108  format (' (CCA) Pourcentage de la variance totale de',
     +        ' Q-barre explique =', F10.5)
 208  format (' (CCA) Percentage of the total variance of Q-bar',
     +        ' accounted for =', F10.5)   
 109  format(/' Tous les coefficients imprimes sont calcules entre'
     +       /' la matrice X centree (en utilisant les poids en ACC)'
     +       /' et la matrice Y centree (ACR) ou transformee en Q-bar',
     +        ' (ACC).')
 209  format(/' All coefficients printed are computed between the'
     +       /' centred matrix X (using weights in CCA) and the'
     +       /' centred (RDA) or transformed to Q-bar (CCA) matrix Y.')
 110  format(/' Les numeros entre parentheses designent les colonnes',
     +        ' (variables) de X'
     +       /' qui sont fusionnees pour former une nouvelle',
     +        ' colonne (variable) de X.'
     +       /' La nouvelle variable remplace les anciennes',
     +        ' dans la suite de l''analyse.')
 210  format(/' Numbers in parentheses indicate the columns',
     +        ' (variables) of X that are'
     +       /' combined during an iteration to form a new column',
     +        ' (variable) of X.'
     +       /' The new column replaces the old ones',
     +        ' in further iterations.')

 111  format(/' Dans chaque ligne de six coefficients,'
     +       /' - le premier est le coefficient de la premiere',
     +        ' variable fusionnee;'
     +       /' - le second est le coefficient de la deuxieme variable',
     +        ' fusionnee;'
     +       /' - le troisieme est le coefficient du produits des deux',
     +        ' variables;'
     +       /' - le quatrieme est le coefficient du carre de la',
     +        ' premiere variable;'
     +       /' - le cinquieme est le coefficient du carre de la',
     +        ' deuxieme variable;'
     +       /' - le sixieme est l''ordonnee a l''origine.')

 211  format(/' In each row of six coefficients,'
     +       /' - the first one is the coefficient of the first',
     +        ' combined variable;'
     +       /' - the second is the coefficient of the second combined',
     +        ' variable;'
     +       /' - the third is coefficient of the product of the two',
     +        ' variables;'
     +       /' - the fourth is the coefficient of the square of the',
     +        ' first variable;'
     +       /' - the fifth is the coefficient of the square of the',
     +        ' second variable;'
     +       /' - the sixth is the intercept.')

 112  format(/' Dans la ligne de deux coefficients obtenus lors de',
     +        ' la derniere iteration'
     +       /' de la procedure polynomiale, le premier est',
     +        ' la constante; le second est le'
     +       /' coefficient de la derniere variable combinee',
     +        ' correspondant a la matrice X(nx1)'
     +       /' lorsqu''elle est reduite a un vecteur.')  
 212  format(/' In the row of two coefficients obtained during the',
     +        ' last iteration of'
     +       /' the polynomial procedure, the first is the intercept;',
     +        ' the second'
     +       /' is the coefficient of the last combined variable',
     +        ' associated with'
     +       /' a matrix X(nx1) which is already reduced to a single',
     +        ' vector.')               
 120  format(/' Approximation de la colonne numero',I3,' de la',
     +        ' matrice Y'/)             
 220  format(/' Approximation of column number',I3,' of',
     +        ' matrix Y'/) 
 121  format( ' Coefficients de l''approximation finale de la',
     +        ' variable',I3,' de Y')
 221  format( ' Coefficients of the final approximation of',
     +        ' variable',I3,' of Y') 
 125  format ('(',$)
 126  format ($,I3,$,',',$)
 127  format ($,I3,$)
 128  format ($,'  ) + (',$)
 129  format ($,I3,$,',',$)   
 130  format ($,I3,$,'  )') 
 427  format ($,7F10.5)
 428  format ($,F10.5)
      end    
      
     
      Subroutine ScaleRDA(n,p,nmax,langue,ichoix3,ican,nvalpr,Qbar,Yhat,
     +           Lambda,SingVal,U,F,Z,vect,LUT,regtype)
      Integer p,regtype
      Real*8 Qbar(nmax,nmax),Yhat(nmax,nmax)
      Real*8 Lambda(nmax),SingVal(nmax),U(nmax,nmax)
      Real*8 F(nmax,nmax)
      Real*8 Z(nmax,nmax),vect(nmax),temp

      temp=dsqrt(dfloat(n-1))
C Cadrage = 1: U, F et Z / Scaling = 1: U, F and Z
C Qbar contient Y centre / Qbar contains centred Y
      if(ican.eq.1) call Produit(Qbar,U,F,n,p,nvalpr,nmax) 
      call Produit(Yhat,U,Z,n,p,nvalpr,nmax) 
      if(ichoix3.eq.1) then
         if(langue.eq.1) write(LUT,*) 
     +      'Cadrage = 1: preserve les distances entre sites'
         if(langue.eq.2) write(LUT,*) 'Scaling = 1: distance biplot'
         else
C Cadrage = 2: U, F et Z / Scaling = 2: U, F and Z
         if(langue.eq.1) write(LUT,*) 
     +      'Cadrage = 2: preserve les correlations entre descripteurs'
         if(langue.eq.2) write(LUT,*) 'Scaling = 2: correlation biplot'
C        Calculer U * Lambda**0.5 / Compute U * Lambda**0.5
         do 22 i=1,p
         do 22 j=1,nvalpr
   22    U(i,j)=U(i,j)*SingVal(j)
C        Recadrer F a variance = 1 / Rescale F to variance = 1
         if(ican.eq.1) then
            do 24 i=1,n
            do 24 j=1,nvalpr
   24       F(i,j)=F(i,j)/SingVal(j)
            endif
C        Recadrer Z a variance = 1 / Rescale Z to variance = 1
         do 26 i=1,n
         do 26 j=1,nvalpr
   26    Z(i,j)=Z(i,j)/SingVal(j)
         endif

C Impression / Print
C     Vecteurs propres / Eigenvectors
      write(LUT,*)
      if(ichoix3.eq.1) then
         if(langue.eq.1) write(LUT,*) 'Vecteurs propres normalises ',
     +                          '(matrice U): positions des especes'
         if(langue.eq.2) write(LUT,*) 'Normalized eigenvectors ',
     +				  '(matrix U): Species scores'
         else
         if(langue.eq.1) write(LUT,*) 'Vecteurs propres',
     +                                ' (U * Lambda**0.5)'
         if(langue.eq.2) write(LUT,*) 'Eigenvectors (U * Lambda**0.5)'
         endif
      write(LUT,*)
      do 10 i=1,p
   10 write(LUT,100) (U(i,j), j=1,nvalpr)
      write(LUT,*)
C     Calculer la longueur des vecteurs propres 
C     Compute length of eigenvectors
      do 28 j=1,nvalpr
   28 vect(j)=0.0
      do 30 i=1,p
      do 30 j=1,nvalpr
   30 vect(j)=vect(j)+U(i,j)**2
      if(langue.eq.1) write(LUT,*) 'Sommes de carres'
      if(langue.eq.2) write(LUT,*) 'Sums of squares'
      write(LUT,100) (vect(j), j=1,nvalpr)
      if(langue.eq.1) write(LUT,*) 'Longueurs'
      if(langue.eq.2) write(LUT,*) 'Lengths'
      write(LUT,100) (dsqrt(vect(j)), j=1,nvalpr)
      write(LUT,*)
C     Position des sites / Site scores
      if(ican.eq.1) then
       if(langue.eq.1) write(LUT,*) 'Position des sites (matrice F)'
       if(langue.eq.2) write(LUT,*) 'Site scores (matrix F)'
       write(LUT,*)
       do 12 i=1,n
   12  write(LUT,100) (F(i,j), j=1,nvalpr)
       write(LUT,*)
C     Calculer la longueur des vecteurs de F
C     Compute length of the vectors of F
         do 32 j=1,nvalpr
   32    vect(j)=0.0
         do 34 i=1,n
         do 34 j=1,nvalpr
   34    vect(j)=vect(j)+F(i,j)**2
         write(LUT,*) 'Variances'
         write(LUT,100) (vect(j)/dfloat(n-1), j=1,nvalpr)
         write(LUT,*)
         endif
C     Position des sites dans l'espace X / Site scores in space X
      if(ican.eq.1) then
         if(langue.eq.1)  then       
                     if(regtype.eq.1) then 
                        write(LUT,*)                   
     +                  'Positions des sites (matrice Z)/comb.',
     +                  ' lineaires des var. explicatives '
                     else
                         write(LUT,*)
     +                  'Positions des sites (matrice Z)/comb.',
     +                  'polynomiales des var. explicatives '                 
                     end if
         else 
                     if(regtype.eq.1) then 
                        write(LUT,*)                   
     +                  'Site scores (matrix Z)/linear comb.',
     +                  ' of explanatory variables '
                     else
                         write(LUT,*)
     +                  'Site scores (matrix Z)/polynomial comb.',
     +                  ' of explanatory variables '                 
                     end if
         end if
         
         else
         if(langue.eq.1) write(LUT,*) 'Position des sites (matrice F)'
         if(langue.eq.2) write(LUT,*) 'Site scores (matrix F)'
         endif
      write(LUT,*)
      do 14 i=1,n
   14 write(LUT,100) (Z(i,j), j=1,nvalpr)
      write(LUT,*)
C     Calculer la longueur des vecteurs de Z 
C     Compute length of the vectors of Z
      do 36 j=1,nvalpr
   36 vect(j)=0.0
      do 38 i=1,n
      do 38 j=1,nvalpr
   38 vect(j)=vect(j)+Z(i,j)**2
      
      if(langue.eq.1) write(LUT,*) 'Pour le diagramme de double ',
     +              'projection, on peut employer soit les ',
     +              'matrices U et F, soit les matrices U et Z'				
      if(langue.eq.2) write(LUT,*) 'A biplot can be drawn using ',
     +		  'either matrices U and F, or matrices U and Z'
      write(LUT,*) 

      write(LUT,*) 'Variances'
      write(LUT,100) (vect(j)/dfloat(n-1), j=1,nvalpr)
      write(LUT,*)
 
C      if(langue.eq.1) write(LUT,*) 'On peut employer l''un ou ',
C     +            'l''autre des ensembles de vecteurs ci-dessous ',
C     +            'pour representer les variables explicatives X ',
C     +            'dans le diagramme de double projection'
C      if(langue.eq.2) write(LUT,*) 'Either of the two following sets ',
C     +            'can be used to represent the explanatory ',
C     +            'variables X in the biplot'
C      write(LUT,*) 

      return
  100 format(8F10.5)
      end


      Subroutine ScaleCCA(n,p,nmax,langue,ichoix3,ican,nvalpr,Qbar,Yhat,
     +           Dipmsq,Dpjmsq,Lambda,SingVal,U,Uhat,UL,V,Vhat,Z,iprint,
     +           LUT,regtype)
      Integer p,regtype
      Real*8 Qbar(nmax,nmax),Yhat(nmax,nmax)
      Real*8 Dipmsq(nmax),Dpjmsq(nmax)
      Real*8 Lambda(nmax),SingVal(nmax),U(nmax,nmax),Uhat(nmax,nmax),
     +       UL(nmax,nmax)
      Real*8 V(nmax,nmax),Vhat(nmax,nmax)
      Real*8 Z(nmax,nmax)

C AFC-ACC: Calcul de Uhat / CA-CCA: Compute Uhat
      do 45 i=1,p
      do 45 j=1,nvalpr
   45 UL(i,j)=U(i,j)/SingVal(j)
      call Produit(Qbar,UL,Uhat,n,p,nvalpr,nmax) 

C Impression des vecteurs propres / Print eigenvectors
      if(iprint.ge.3) then
         if(langue.eq.1) write(LUT,*) 'Vecteurs propres normalises (U)'
         if(langue.eq.2) write(LUT,*) 'Normalized eigenvectors (U)'
         write(LUT,*)
         do 48 i=1,p
   48    write(LUT,100) (U(i,j), j=1,nvalpr)
         write(LUT,*)
         if(langue.eq.1) write(LUT,*) 'Vecteurs propres normalises (U^)'
         if(langue.eq.2) write(LUT,*) 'Normalized eigenvectors (U^)'
         write(LUT,*)
         do 50 i=1,n
   50    write(LUT,100) (Uhat(i,j), j=1,nvalpr)
         write(LUT,*)
         endif
C Cadrage en ACC / Scaling in CCA
C
C Coordonnees des especes, cadrage = 1 
C Species scores, scaling = 1: V = Dpj-0.5 * U
C   Dpjmsq contient Dpj-0.5
      do 52 i=1,p
      do 52 j=1,nvalpr
   52 V(i,j)=U(i,j)*Dpjmsq(i)

C Coordonnees des sites, cadrage = 2 
C Site scores, scaling = 2: V^ = Dip-0.5*U^
C (Inutile pour l'analyse des residus) 
C (Useless in the analysis of residuals)
C Dipmsq contient Dip-0.5
C Uhat contient U^
      if(ican.eq.1) then
         do 56 i=1,n
         do 56 j=1,nvalpr
   56    Vhat(i,j)=Uhat(i,j)*Dipmsq(i)
         endif

C Cadrage = 1 / Scaling = 1:
      if(ichoix3.eq.1) then
         if(langue.eq.1) write(LUT,*) 'Cadrage = 1:',
     +                                ' sites au centroide des especes'
         if(langue.eq.2) write(LUT,*) 'Scaling = 1: sites at',
     +                                ' centroids of species'
         write(LUT,*)
C     Lignes / Site scores: F = V^ * Lambda+0.5
C     (Inutile pour l'analyse des residus) 
C     (Useless in the analysis of residuals)
C     C contient les valeurs propres / C contains the eigenvalues
         if(ican.eq.1) then
            do 58 i=1,n
            do 58 j=1,nvalpr
   58       Vhat(i,j)=Vhat(i,j)*SingVal(j)
            endif

C     Lignes CLVE / Site scores LCEV: Z = Dip-0.5 * Y^ * U
C     Yhat contient Y^ / Yhat contains Y^
C     Dipmsq contient Dip-0.5 / Dipmsq contains Dip-0.5
         call Produit(Yhat,U,Z,n,p,nvalpr,nmax) 
         do 64 i=1,n
         do 64 j=1,nvalpr
   64    Z(i,j)=Z(i,j)*Dipmsq(i)
C     Impression: V, F, Z / Print V, F, Z
         if(langue.eq.1) write(LUT,*) 'Colonnes (especes, matrice V)'
         if(langue.eq.2) write(LUT,*) 'Species scores (matrice V)'
         write(LUT,*)
         do 54 i=1,p
   54    write(LUT,100) (V(i,j), j=1,nvalpr)
         write(LUT,*)

         if(ican.eq.1) then
            if(langue.eq.1) write(LUT,*) 'Lignes (sites, matrice F)'
            if(langue.eq.2) write(LUT,*) 'Site scores (matrice F)'
            write(LUT,*)
            do 60 i=1,n
   60       write(LUT,100) (Vhat(i,j), j=1,nvalpr)
            write(LUT,*)
            endif

         if(ican.eq.1) then
         
         
            if(langue.eq.1) then         
                     if(regtype.eq.1) then 
                        write(LUT,*)                   
     +                  'Lignes (sites, matrice Z)/ comb. lineaires',
     +                  ' des var. explicatives'
                     else
                         write(LUT,*)
     +                  'Lignes (sites, matrice Z)/ comb.',
     +                  ' polynomiales des var. explicatives'                 
                     end if
            else 
                     if(regtype.eq.1) then 
                        write(LUT,*)                   
     +                  'Site scores (matrice Z)/ linear comb.',
     +                  ' of explanatory variables'
                     else
                         write(LUT,*)
     +                  'Site scores (matrice Z)/ polynomial comb.',
     +                  ' of explanatory variables'               
                     end if
            end if
                     
         else
            if(langue.eq.1) write(LUT,*) 'Lignes (sites, matrice F)'
            if(langue.eq.2) write(LUT,*) 'Site scores (matrice F)'
            endif
         write(LUT,*)
         do 66 i=1,n
   66    write(LUT,100) (Z(i,j), j=1,nvalpr)
         write(LUT,*)
C Cadrage = 2 / Scaling = 2:
      else
         if(langue.eq.1) write(LUT,*) 'Cadrage = 2:',
     +                                ' especes au centroide des sites'
         if(langue.eq.2) write(LUT,*) 'Scaling = 2: species at',
     +                                ' centroids of sites'
         write(LUT,*)
C     Colonnes / Species scores: F^ = V * Lambda+0.5
C     Lambda contient les valeurs propres 
C     Lambda contains the eigenvalues
         do 68 i=1,p
         do 68 j=1,nvalpr
   68    V(i,j)=V(i,j)*SingVal(j)

C     Lignes CLVE / Site scores LCEV: Z = Dip-0.5 * Y^ * U * lambda-0.5
C     Yhat contient Y^ / Yhat contains Y^
C     UL contient (U * lambda-0.5) / UL contains (U * lambda-0.5)
C     Dipmsq contient Dip-0.5 / Dipmsq contains Dip-0.5
         call Produit(Yhat,UL,Z,n,p,nvalpr,nmax) 
         do 72 i=1,n
         do 72 j=1,nvalpr
   72    Z(i,j)=Z(i,j)*Dipmsq(i)
C     Impression: Fhat, Vhat, Z / Print Fhat, Vhat, Z
         if(langue.eq.1) write(LUT,*) 'Colonnes (especes, matrice F^)'
         if(langue.eq.2) write(LUT,*) 'Species scores (matrice F^)'
         write(LUT,*)
         do 70 i=1,p
   70    write(LUT,100) (V(i,j), j=1,nvalpr)
         write(LUT,*)
         if(ican.eq.1) then
            if(langue.eq.1) write(LUT,*) 'Lignes (sites, matrice V^)'
            if(langue.eq.2) write(LUT,*) 'Site scores (matrice V^)'
            write(LUT,*)
            do 62 i=1,n
   62       write(LUT,100) (Vhat(i,j), j=1,nvalpr)
            write(LUT,*)
            endif
         if(ican.eq.1) then
            
            if(langue.eq.1) then        
                     if(regtype.eq.1) then 
                        write(LUT,*)                   
     +                  'Lignes (sites, matrice Z)/ comb. lineaires',
     +                  ' des var. explicatives'
                     else
                         write(LUT,*)
     +                  'Lignes (sites, matrice Z)/ comb. polynomiales',
     +                  ' des var. explicatives'                 
                     end if
            else 
                     if(regtype.eq.1) then 
                        write(LUT,*)                   
     +                  'Site scores (matrice Z)/ linear comb.',
     +                  ' of explanatory variables '
                     else
                         write(LUT,*)
     +                  'Site scores (matrice Z)/ polynomial comb.',
     +                  ' of explanatory variables '               
                     end if
            end if
         
            else
            if(langue.eq.1) write(LUT,*) 'Lignes (sites, matrice V^)'
            if(langue.eq.2) write(LUT,*) 'Site scores (matrice V^)'
            endif
         write(LUT,*)
         do 74 i=1,n
   74    write(LUT,100) (Z(i,j), j=1,nvalpr)
         write(LUT,*)
      endif
      return
  100 format(8F10.5)
      end
      

      Subroutine TransRDA(n,p,m,nmax,Y,X,Variable01,totinert,
     +                    sx,iprint,langue,LUT,igo)
      Integer p,Variable01(nmax)
      Real*8 Y(nmax,nmax),X(nmax,nmax)
      Real*8 sx(nmax),totinert,dfln,dflnm1
C Transformation des matrices Y et X
C avant l'ACP ou l'analyse de redondance
C Transformation of matrices Y and X prior to PCA or redundancy analysis
C
C Centrage des colonnes de Y / Centring the column vectors of Y
      dfln=dfloat(n)
      do 8 j=1,p
    8 sx(j)=0.0
      do 10 i=1,n
      do 10 j=1,p
   10 sx(j)=sx(j)+Y(i,j)
      do 12 j=1,p
   12 sx(j)=sx(j)/dfln
      do 14 i=1,n
      do 14 j=1,p
   14 Y(i,j)=Y(i,j)-sx(j)

C Calcul de l'inertie totale de Y / Compute total inertia in Y
      totinert=0.0
      do 16 i=1,n
      do 16 j=1,p
      totinert=totinert+Y(i,j)**2
   16 continue
      
      if(igo.eq.2) return
      
C Centrage des variables de la matrice X 
C Centring the variables in matrix X
      do 20 j=1,m-1
   20 sx(j)=0.0
      do 22 i=1,n
      do 22 j=1,m-1
   22 sx(j)=sx(j)+X(i,j)
      do 24 j=1,m-1
   24 sx(j)=sx(j)/dfln
      do 28 j=1,m-1
      if (Variable01(j).eq.0) then
         do 26 i=1,n
   26    X(i,j)=X(i,j)-sx(j)
         endif
   28 continue        
C
      if((iprint.ne.1).and.(iprint.ne.4)) return      
C
      if(langue.eq.1) write(LUT,*) 'Matrice Y centree'
      if(langue.eq.2) write(LUT,*) 'Centred matrix Y'
      write(LUT,*)
      do 32 i=1,n
   32 write(LUT,100) (Y(i,j), j=1,p)
      write(LUT,*)
      if(m.eq.0) return
      if(langue.eq.1) write(LUT,*) 'Matrice X centree'
      if(langue.eq.2) write(LUT,*) 'Matrix X centred'
      write(LUT,*)
      do 34 i=1,n
   34 write(LUT,100) (X(i,k), k=1,m-1)
      write(LUT,*)
      return
  100 format(8F10.5)
      end

      Subroutine TransCCA(n,p,m,nmax,Y,Qbar,X,Variable01,Fip,Dip,Dpj,
     +                    Ftot,temp,sx,iprint,langue,LUT)
      Integer p,Variable01(nmax)
      Real*8 Y(nmax,nmax),Qbar(nmax,nmax),X(nmax,nmax)
      Real*8 Fip(nmax),Dip(nmax),Dpj(nmax),Ftot
      Real*8 sx(nmax),temp
C Transformation de Y et X pour l'AFC ou l'ACC
C Transformation of Y and X prior to CA or CCA
      do 2 i=1,n
    2 Fip(i)=0.0
      do 4 j=1,p
    4 Dpj(j)=0.0
      Ftot=0.0
      do 6 i=1,n
      do 6 j=1,p
      Fip(i)=Fip(i)+Y(i,j)
    6 Dpj(j)=Dpj(j)+Y(i,j)
      do 8 j=1,p
    8 Ftot=Ftot+Dpj(j)
      do 9 j=1,p
    9 Dpj(j)=Dpj(j)/Ftot
      do 10 i=1,n
   10 Dip(i)=Fip(i)/Ftot
C Transformation de Y en Q=Y/Ftot et Qbar 
C Transformation of Y into Q=Y/Ftot and Qbar
      do 14 i=1,n
      do 14 j=1,p
      temp=dsqrt(Dip(i)*Dpj(j))
   14 Qbar(i,j)=(Y(i,j)/Ftot - Dip(i)*Dpj(j))/temp

C Calcul de l'inertie totale de Qbar / Compute total inertia in Qbar
      temp=0.0
      do 16 i=1,n
      do 16 j=1,p
      temp=temp+(Qbar(i,j))**2
   16 continue
C      
C Centrage des variables de la matrice X
C Centring the variables in matrix X
      do 20 j=1,m-1
   20 sx(j)=0.0
      do 22 i=1,n
      do 22 j=1,m-1
   22 sx(j)=sx(j)+Fip(i)*X(i,j)
      do 24 j=1,m-1
   24 sx(j)=sx(j)/Ftot
      do 28 j=1,m-1
      if (Variable01(j).eq.0) then
         do 26 i=1,n
   26    X(i,j)=X(i,j)-sx(j)
         endif
   28 continue        
C
      if((iprint.ne.1).and.(iprint.ne.4)) return

      write(LUT,*)
      if(langue.eq.1) write(LUT,*) 'Matrice Y transformee (Q-bar)'
      if(langue.eq.2) write(LUT,*) 'Transformed Y matrix (Q-bar)'
      write(LUT,*)
      do 32 i=1,n
   32 write(LUT,100) (Qbar(i,j), j=1,p)
      write(LUT,*)
      if(m.eq.0) return
      if(langue.eq.1) write(LUT,*) 'Matrice X centree'
      if(langue.eq.2) write(LUT,*) 'Matrix X centred'
      write(LUT,*)
      do 34 i=1,n
   34 write(LUT,100) (X(i,k), k=1,m-1)
      write(LUT,*)
      return
  100 format(8F10.5)
      end
         

C                    ----------------------------

C The following subroutines are modified from Numerical Recipes.
C
C The authors of the present program are not authorized to distribute
C the source code of Numerical Recipes subroutines. If you want
C to use this subroutine to recompile the present program
C (in the original or some modified form) or in 
C any other program, you should obtain a license from Numerical Recipes. 
C See the WWWeb site <http://world.std.com/~nr/storefront.html>.
C The subroutine is reproduced here to show how the original 
C Numerical Recipes subroutine should be modified before incorporating
C it in the present program.
C

      Subroutine GaussJ(A,nn)
      INCLUDE 'Rdparam.f'
      Parameter (mp=1,n=nmax)
      Real*8 A(n,n)
      Real*8 AA(n,n),B(n,mp),val,trace,big,pivinv,dum,BB(n,n)
      Integer ipiv(n), indxr(n), indxc(n)
c-----------------------------------------------------------------------
c Linear equation solution by Gauss-Jordan elimination.
c From 'Numerical Recipes' (Press et al., 1986), p. 28-29.
c A is an input matrix of 'nn x nn' elements, stored in a array 
c of physical dimensions 'n x n'. B is an input matrix of 'nn x m'
c containing the m right-hand side vectors, stored in an array of
c physical dimension 'n x mp'.
c (In this implementation, B only contains a column vector of 
c zero on start.) On output, A is replaced by its matrix inverse, 
c and B is replaced by the corresponding set of solution vectors.
c 
c Note: a singular matrix causes an appropriate "pause" message. 
c 
c The integer arrays IPIV, INDXR and INDXC are used for bookkeeping
c on the pivoting. 'n' should be as large as the largest anticipated 
c value of 'nn'.
c
c Modified for GSLIB from Numerical Recipes (1986),
c Section 2.1, p. 28-29.
c 
C Solves the system: [B(nxm)|identity] = A(nxn).[X(nxm)|invA] where m=1.
c X (stored in B on input) is the initial solution vector of the set of
c linear equations; on output, B is the set of regression coefficients.
c
c Pour realiser une regression lineaire:
c au depart, B est le vecteur des valeurs (X'y).
c A est la matrice [X'X] a inverser.
C A la fin du sous-programme, B contient les coefficients de regression.
C
C For linear regression:
C At the beginning, B contains the vector (X'y).
C A contains the matrix [X'X] to be inverted.
C At the end of the subprogram, B contains the regression coefficients.
c-----------------------------------------------------------------------
      m=1
      np=n
      if (np .gt. n .or. mp .gt. n) stop 'dim of Gaussj exceeded.'
      if (nn .lt. 1 .or. m .lt. 1) stop 'illogical nn or m in Gaussj.'
      if (nn .gt. nn .or. m .gt. mp) stop 'nn or m too large in Gaussj'
C     Copie de A -> AA.
      do 4 i=1,nn
      do 2 j=1,m
    2 b(i,j)=0.0
      do 4 j=1,nn
      AA(i,j)=A(i,j)
    4 continue 
C
      do 10 j = 1, nn
      ipiv(j) = 0
   10 continue 
C Beginning the main loop (110) over the columns to be reduced
      do 110 i = 1, nn
        big = 0.
C Beginning the outer loop (30) of the search for a pivot element
        do 30 j = 1, nn
          if (ipiv(j) .ne. 1) then 
            do 20 k = 1, nn
              if (ipiv(k) .eq. 0) then 
                if (abs(a(j,k)) .ge. big) then 
                  big = abs(a(j,k))
                  irow = j
                  icol = k
                end if 
              else if (ipiv(k) .gt. 1) then 
                pause 'singular matrix'
              end if 
   20       continue 
          end if 
   30   continue 
        ipiv(icol) = ipiv(icol) + 1
C We now have the pivot element, so we interchange rows, if needed, 
C to put the pivot element on the diagonal. The columns are not 
C physically interchanged, only relabeled: INDX(i), the column of
C the i-th pivot, isthe i-th column that is reduced, while INDXR(i)
C is the row in which thatpivot element was originally located. 
C If INDXR(i)<>INDXC(i) there is animplied column interchange.  
C With this form of bookkeeping, the solution B's will end up in 
C the correct order, and the inverse matrix will bescrambled by columns.
        if (irow .ne. icol) then 
          do 40 l = 1, nn
            dum = a(irow,l)
            a(irow,l) = a(icol,l)
            a(icol,l) = dum
   40     continue 
          do 50 l = 1, m
            dum = b(irow,l)
            b(irow,l) = b(icol,l)
            b(icol,l) = dum
   50     continue 
        end if 
        indxr(i) = irow
        indxc(i) = icol
C We are now ready to divide the pivot row by the pivot element, 
C located at Irow and Icol
        if (a(icol,icol) .eq. 0.) pause 'singular matrix.'
        pivinv = 1. / a(icol,icol)
        a(icol,icol) = 1.
        do 60 l = 1, nn
          a(icol,l) = a(icol,l) * pivinv
   60   continue 
        do 70 l = 1, m
          b(icol,l) = b(icol,l) * pivinv
   70   continue 
C Next we reduce the rows, except for the pivot one, of course
        do 100 ll = 1, nn
          if (ll .ne. icol) then 
            dum = a(ll,icol)
            a(ll,icol) = 0.
            do 80 l = 1, nn
              a(ll,l) = a(ll,l) - a(icol,l) * dum
   80       continue 
            do 90 l = 1, m
              b(ll,l) = b(ll,l) - b(icol,l) * dum
   90       continue 
          end if 
  100   continue 
  110 continue 
C End of the main loop over columns of the reduction. 
C It only remains to unscramble the solution in view of the column
C interchanges. We do this by interchanging pairs of columns in the
C reverse order that the permutation was built up.
      do 130 l = nn, 1, -1
        if (indxr(l) .ne. indxc(l)) then 
          do 120 k = 1, nn
            dum = a(k,indxr(l))
            a(k,indxr(l)) = a(k,indxc(l))
            a(k,indxc(l)) = dum
  120     continue 
        end if 
  130 continue 
C
C End of inversion procedure.
      return
      end


      Function GaussJ1 (A,nn)
C Si la matrice A n'est pas singuliere, la fonction retourne
C la valeur 0. Dans ce cas, la fonction fournit la matrice inversee  
C de A (a la place de A). Si A est singuliere, alors la fonction 
C retourne la valeur 1.
C
C If A is not a singular matrix, then the function return the value 0.  
C In this case the function also provides the inverse matrix of A 
C (in place of A). If A is a singular matix, then the function return
C the value 1  
      INCLUDE 'Rdparam.f' 
      Parameter (mp=1,n=nmax)
      Real*8 A(n,n)
      Real*8 AA(n,n),B(n,mp),val,trace,big,pivinv,dum
      Integer ipiv(n), indxr(n), indxc(n)

      GaussJ1=0     
      m=1
      np=n
      if (np .gt. n .or. mp .gt. n) stop 'dim of Gaussj exceeded.'
      if (nn .lt. 1 .or. m .lt. 1) stop 'illogical nn or m in Gaussj.'
      if (nn .gt. nn .or. m .gt. mp) stop 'nn or m too large in Gaussj'
C     Copie de A -> AA.
      do 4 i=1,nn
      do 2 j=1,m
    2 b(i,j)=0.0
      do 4 j=1,nn
      AA(i,j)=A(i,j)
    4 continue 
C
      do 10 j = 1, nn
      ipiv(j) = 0
   10 continue 
C Beginning the main loop (110) over the columns to be reduced
      do 110 i = 1, nn
        big = 0.
C Beginning the outer loop (30) of the search for a pivot element
        do 30 j = 1, nn
          if (ipiv(j) .ne. 1) then 
            do 20 k = 1, nn            
              if (ipiv(k) .eq. 0) then 
                if (abs(a(j,k)) .ge. big) then 
                  big = abs(a(j,k))
                  irow = j
                  icol = k
                end if 
              else if (ipiv(k) .gt. 1) then                       
                      GaussJ1=1
                      return 
                   end if 
   20       continue 
          end if 
   30   continue 
        ipiv(icol) = ipiv(icol) + 1
C We now have the pivot element, so we interchange rows, if needed, to 
C put the pivot element on the diagonal. The columns are not physically
C interchanged, only relabeled: INDX(i), the column of the i-th pivot,
C is the i-th column that is reduced, while INDXR(i) is the row in which
C that pivot element was originally located. If INDXR(i)<>INDXC(i) there
C is animplied column interchange. With this form of bookkeeping, 
C the solution B's will end up in the correct order, and the inverse
C s matrix will becrambled by columns.
C        write(*,*) 'Test2'
        if (irow .ne. icol) then 
          do 40 l = 1, nn
            dum = a(irow,l)
            a(irow,l) = a(icol,l)
            a(icol,l) = dum
   40     continue 
          do 50 l = 1, m
            dum = b(irow,l)
            b(irow,l) = b(icol,l)
            b(icol,l) = dum
   50     continue 
        end if 
        indxr(i) = irow
        indxc(i) = icol
C We are now ready to divide the pivot row by the pivot element, 
C located at Irow and Icol
C        write (*,*) icol, a(icol,icol)
        if (dabs(a(icol,icol)) .le. 0.0000001) then
                GaussJ1=1
                return 
        end if
        pivinv = 1. / a(icol,icol)
        a(icol,icol) = 1.
        do 60 l = 1, nn
          a(icol,l) = a(icol,l) * pivinv
   60   continue 
        do 70 l = 1, m
          b(icol,l) = b(icol,l) * pivinv
   70   continue 
C Next we reduce the rows, except for the pivot one, of course
        do 100 ll = 1, nn
          if (ll .ne. icol) then 
            dum = a(ll,icol)
            a(ll,icol) = 0.
            do 80 l = 1, nn
              a(ll,l) = a(ll,l) - a(icol,l) * dum
   80       continue 
            do 90 l = 1, m
              b(ll,l) = b(ll,l) - b(icol,l) * dum
   90       continue 
          end if 
  100   continue 
  110 continue 
C End of the main loop over columns of the reduction. 
C It only remains to unscramble the solution in view of the column
C interchanges. We do this by interchanging pairs of columns in the
C reverse order that the permutation was built up.
      do 130 l = nn, 1, -1
        if (indxr(l) .ne. indxc(l)) then 
          do 120 k = 1, nn
            dum = a(k,indxr(l))
            a(k,indxr(l)) = a(k,indxc(l))
            a(k,indxc(l)) = dum
  120     continue 
        end if 
  130 continue 
C
C End of inversion procedure.
      return
      end     


      Function Pythag(a,b)
      Real*8 a,b,pythag
      Real*8 absa,absb
C Computes sqrt(a^2 + b^2) without destructive underflow or overflow
C From 'Numerical Recipes' (Press et al., 1986).
      absa=abs(a)
      absb=abs(b)
      if(absa.gt.absb)then
           pythag=absa*sqrt(1.+(absb/absa)**2)
      else
           if(absb.eq.0.)then
                pythag=0.
           else
                pythag=absb*sqrt(1.+(absa/absb)**2)
           endif
      endif
      return
      end


C Following are two subroutines for finding the eigenvalues of a real
C symmetric matrix using the Householder method, a procedure recommended
C by Numerical Recipes (Press et al., 1986): routines TRED2 and TQLI.

      Subroutine TRED2(A,N,NP,D,E)
C
C Householder reduction of a real, symmetric, N by N matrix A, stored in
C an NP by NP physical array.  On output, A is replaced by the
C orthogonal matrix Q effecting the transformation. D returns 
C the diagonal elements of the tridiagonal matrix, and E the 
C off-diagonal elements, with E(1) = 0. Several statements, 
C as noted in comments, can be omitted if onlyeigenvalues are to be
C found, in which case A contains no useful information on output. 
C Otherwise they are to be included.

      Dimension A(NP,NP), D(NP), E(NP)
C REAL*8 statement added for double precision.
      Real*8 A,D,E,H,SCALE,F,G,HH
      
      IF (N.GT.1) THEN
         DO 18 I = N,2,-1
              L = I - 1
              H = 0.
              SCALE = 0.
              IF (L.GT.1) THEN
                 DO 11 K = 1,L
                    SCALE = SCALE + DABS(A(I,K))
11                  CONTINUE
              IF (SCALE.EQ.0) THEN
                 E(I) = A(I,L)
              ELSE
                 DO 12 K = 1,L
                    A(I,K) = A(I,K)/SCALE
                    H = H + A(I,K)**2
12               CONTINUE
                 F = A(I,L)
                 G = -SIGN(DSQRT(H),F)
                 E(I) = SCALE*G
                 H = H-F*G
                 A(I,L) = F-G
                 F = 0.
                 DO 15 J = 1,L
C Omit following line if finding only eigenvalues.
                    A(J,I) = A(I,J)/H
                    G = 0.
                    DO 13 K = 1,J
                       G = G + A(J,K) * A(I,K)
13                  CONTINUE
                    IF (L.GT.J) THEN
                       DO 14 K = J+1, L
                          G = G + A(K,J)*A(I,K)
14                     CONTINUE
                    END IF
                    E(J) = G/H
                    F = F + E(J)*A(I,J)
15                  CONTINUE
                 HH = F/(H + H)
                 DO 17 J = 1,L
                    F = A(I,J)
                    G = E(J) - HH*F
                    E(J) = G
                    DO 16 K = 1,J
                       A(J,K) = A(J,K) - F*E(K) - G*A(I,K)
16                  CONTINUE
17               CONTINUE
              END IF
              ELSE
                 E(I) = A(I,L)
              END IF
              D(I) = H
18       CONTINUE
      END IF
C Omit following line if finding only eigenvalues.
      D(1) = 0.
      E(1) = 0.
      DO 23 I = 1,N
C Delete lines from here...
         L = I-1
         IF (D(I).NE.0.) THEN
            DO 21 J = 1,L
               G = 0.
               DO 19 K = 1,L
                  G = G + A(I,K) * A(K,J)
19             CONTINUE
               DO 20 K = 1,L
                  A(K,J) = A(K,J) - G*A(K,I)
20             CONTINUE
21          CONTINUE
         END IF
C ... to here when finding only eigenvalues.
         D(I) = A(I,I)
C Also delete lines from here ....
         A(I,I) = 1.
         IF(L.GE.1) THEN
            DO 22 J = 1,L
               A(I,J) = 0.
               A(J,I) = 0.
22          CONTINUE
         END IF
C ... to here when finding only eigenvalues.
23    continue
      return
      end

      Subroutine TQLI (D,E,N,NP,Z)
C
C QL algorithm with implicit shifts, to determine the eigenvalues and
C eigenvectors of a real, symmetric, tridiagonal matrix previously
C reduced by TRED 2 (Numerical Recipes).  D is a vector of length NP.
C On input, its first N elements are the diagonal elements of the 
C tridiagonal matrix. On output, it returns the eigenvalues.
C The vector E inputs the subdiagonal elements of the tridiagonal 
C matrix,with E(1) arbitrary. On output, E is destroyed. When
C finding only the eigenvalues, several lines may be omitted,
C as noted in the coments. If the eigenvectors of the tridiagonal
C matrix are desired, the matrix Z (N by N matrix stored in NP by
C NP array) is input as the identity matrix. If the eigenvectors of
C a matrix that has been reduced by TRED2 are required, then Z is 
C input as the matrix output by TRED2.  In either case, the Kth 
C column of Z returns the normalized eigenvector corresponding to D(K).
C
      Dimension D(NP), E(NP), Z(NP,NP)
C REAL*8 statement added to original code for double precision.
      Real*8 D,E,Z,DD,G,R,S,C,P,F,B

      IF (N.GT.1) THEN
         DO 11 I = 2,N
            E(I-1) = E(I)
11       CONTINUE
         E(N) = 0.
         DO 15 L = 1,N
            ITER = 0
1           DO 12 M = L, N-1
               DD = DABS(D(M)) + DABS(D(M+1))
               IF (DABS(E(M)) + DD.EQ.DD) GO TO 2
12          CONTINUE
            M = N
2           IF (M.NE.L) THEN
               IF (ITER.EQ.50) Stop 'Too many iterations in TQLI'
               ITER = ITER + 1
               G = (D(L+1)-D(L))/(2.*E(L))
               R = SQRT(G**2+1.)
               G = D(M) - D(L) + E(L)/(G + SIGN(R,G))
               S = 1.
               C = 1.
               P = 0.
               DO 14 I = M-1,L,-1
                  F = S*E(I)
                  B = C*E(I)
                  IF (DABS(F).GE.DABS(G)) THEN
                     C = G/F
                     R = DSQRT(C**2+1.)
                     E(I+1) = F*R
                     S = 1./R
                     C = C*S
                  ELSE
                     S = F/G
                     R = DSQRT(S**2+1.)
                     E(I+1) = G*R
                     C = 1./R
                     S = S*C
                  END IF
                  G = D(I+1) - P
                  R = (D(I) - G)*S+2.*C*B
                  P = S*R
                  D(I+1) = G + P
                  G = C*R - B
C Omit lines from here ...
                  DO 13 K = 1,N
                     F = Z(K,I+1)
                     Z(K,I+1) = S*Z(K,I) + C*F
                     Z(K,I) = C*Z(K,I) - S*F
13                CONTINUE
C ... to here when finding only eigenvalues.
14             CONTINUE
               D(L) = D(L) - P
               E(L) = G
               E(M) = 0.
               GO TO 1
            END IF
15          CONTINUE
      end if
      return
      end
